/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"
#include "core/settings.h"
#include "core/utils.h"
#include <QDir>

Config::Config() :
    mConfigPath(QString("~/.config/com.ubuntu.telegram").replace("~",QDir::homePath())),
    mDownloadsPath(QString("~/.cache/com.ubuntu.telegram").replace("~",QDir::homePath())),
    mPublicKeyFile("tg.pug"),
    mDatabasePath("") {
}

Config::~Config() {
}

QString Config::phoneNumber() const {
    return mPhoneNumber;
}

QString Config::configPath() const {
    return mConfigPath;
}

QString Config::publicKeyFile() const {
    return mPublicKeyFile;
}

void Config::setPhoneNumber(const QString &phoneNumber) {
    mPhoneNumber = Utils::parsePhoneNumberDigits(phoneNumber);
}

void Config::setConfigPath(const QString &configPath) {
    mConfigPath = QString(configPath).replace("~",QDir::homePath());
}

void Config::setPublicKeyFile(const QString &publicKeyFile) {
    mPublicKeyFile = publicKeyFile;
}

void Config::setDatabasePath(const QString &databasePath) {
    mDatabasePath = databasePath;
}

void Config::setDownloadsPath(const QString &downloadsPath) {
    mDownloadsPath = QString(downloadsPath).replace("~",QDir::homePath());
}

QString Config::databasePath() {

    if (!mDatabasePath.length()) {
        mDatabasePath = mDownloadsPath;
    }

    QString completePath = QString(mDatabasePath).replace("~",QDir::homePath());

    if (mPhoneNumber.length() > 0) {
       completePath += QDir::separator() + mPhoneNumber;
    }

    QDir databaseDir = QDir(completePath);
    if (!databaseDir.exists()) {
        if (QDir::root().mkpath(completePath)) {
            qCDebug(TG_PLUGIN_LOGIC) << completePath << "database directory created";
        } else {
            qCCritical(TG_PLUGIN_LOGIC) << "error creating database directory: " << completePath;
            return "";
        }
    }

    return completePath;
}

QString Config::downloadsPath() const  {
    return mDownloadsPath + QDir::separator() + mPhoneNumber;
}
