/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DBMANAGER_H
#define DBMANAGER_H

#include <QLoggingCategory>
#include <QObject>
#include <QtSql/QSqlDatabase>

Q_DECLARE_LOGGING_CATEGORY(LOG_DATABASE)

class DbManager : public QObject
{
    Q_OBJECT
public:
    explicit DbManager(QObject *parent = 0);
    ~DbManager();

    bool initialize(const QString &path);
    void close();
    void reopen();
    QSqlDatabase database() const;

    bool beginTransaction();
    bool finishTransaction();
    bool rollbackTransaction();

Q_SIGNALS:
    void initialized();
    void upgraded(qint32 version);

protected:
    bool createOrUpgrade(qint32 currentVersion);
    qint32 getDatabaseVersion();
    void setDatabaseVersion(qint32 version);
    bool executeSqlQueries(const QStringList &queries);

    bool create();
    bool purge();
    bool purgeButSecret(); // removes all database data except the one related with secret chats that would be missed if deleted
    void upgrade(qint32 oldVersion, qint32 newVersion);

    bool upgradeMessageActionsRemovePhotoId();
    bool upgradeMessagesRandomId();
    bool upgradeGroupChat();
    bool upgradeDialogsMuted();
    bool upgradeDialogsDate();
    bool upgradeSecretChats();
    bool upgradeReadHistory();
    bool upgradeEncryptedMediaKeys();
    bool upgradeEncryptedTtl();
    bool upgradeProfilesBigPhotos();

private:
    QString mDatabasePath;
    QSqlDatabase mDatabase;
    qint32 mTransactionsDepth;
};

#endif // DBMANAGER_H
