#include "incomingfile.h"

IncomingFile::IncomingFile(const QString& downloadsPath) :
    mId(0),
    mMediaId(0),
    mIncomingFileType(MessagePhoto),
    mMediaType(MessageMedia::typeMessageMediaEmpty),
    mFileLocationId(0),
    mSizeType(""),
    mIsThumbnail(false),
    mFilename(""),
    mFilePath(""),
    mModificationTime(0),
    mExtension(""),
    mExpectedSize(0),
    mDownloadsPath(downloadsPath) {
}

void IncomingFile::setId(qint32 id) {
    mId = id;
}

void IncomingFile::setMediaId(qint64 mediaId) {
    mMediaId = mediaId;
}

void IncomingFile::setIncomingFileType(IncomingFileType type) {
    mIncomingFileType = type;
}

void IncomingFile::setMediaType(MessageMedia::MessageMediaType mediaType) {
    mMediaType = mediaType;
}

void IncomingFile::setFileLocationId(qint32 fileLocationId) {
    mFileLocationId = fileLocationId;
}

void IncomingFile::setSizeType(const QString &sizeType) {
    mSizeType = sizeType;
}

void IncomingFile::setIsThumbnail(bool isThumbnail) {
    mIsThumbnail = isThumbnail;
}

void IncomingFile::setFilename(const QString &filename) {
    mFilename = filename;
}

void IncomingFile::setModificationTime(qint32 mTime) {
    mModificationTime = mTime;
}

void IncomingFile::setExtension(const QString &extension) {
    mExtension = extension;
}

void IncomingFile::setServerLocation(const FileLocation &serverLocation) {
    mServerLocation = serverLocation;
}

void IncomingFile::setExpectedSize(qint64 expectedSize) { mExpectedSize = expectedSize; }

qint32 IncomingFile::id() const {
    return mId;
}

qint64 IncomingFile::mediaId() const {
    return mMediaId;
}

IncomingFile::IncomingFileType IncomingFile::incomingFileType() const {
    return mIncomingFileType;
}

MessageMedia::MessageMediaType IncomingFile::mediaType() const {
    return mMediaType;
}

qint32 IncomingFile::fileLocationId() const {
    return mFileLocationId;
}

QFile &IncomingFile::file() {
    return mFile;
}

FileLocation IncomingFile::serverLocation() const {
    return mServerLocation;
}

qint32 IncomingFile::expectedSize() const {
    return mExpectedSize;
}

QString IncomingFile::sizeType() const {
    return mSizeType;
}

bool IncomingFile::isThumbnail() const {
    return mIsThumbnail;
}

QString IncomingFile::downloadsPath() const {
    return mDownloadsPath;
}

QString IncomingFile::filePath() {
    if (!mFilePath.length()) {

        mFilePath = mDownloadsPath + QDir::separator();

        QString section;
        switch (mIncomingFileType) {
        case UserThumbnail:
            section = PROFILES_MEDIA_FOLDER;
            mSizeType = "small";
            break;
        case UserPhoto:
            section = PROFILES_MEDIA_FOLDER;
            mSizeType = "big";
            break;
        case ChatThumbnail:
            section = PROFILES_MEDIA_FOLDER;
            mSizeType = "small";
            break;
        case ChatPhoto:
            section = PROFILES_MEDIA_FOLDER;
            mSizeType = "big";
            break;
        case MessagePhoto:
        case EncryptedMessagePhoto:
        case MessageActionPhoto:
            section = PHOTOS_MEDIA_FOLDER;
            break;
        case EncryptedMessageVideo:
        case MessageVideo:
            section = VIDEOS_MEDIA_FOLDER;
            break;
        default:
            break;
        }

        mFilePath += section + QDir::separator();

        if (mSizeType.length()) {
            mFilePath += mSizeType + QDir::separator();
        }

        mFilePath += mMediaId ? QString::number(mMediaId) : QString::number(mId) + "_" + QString::number(mModificationTime);
        mFilePath += mExtension;
    }
    return mFilePath;
}
