/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Roberto Mier
 *      Tiago Herrmann
 */
#ifndef INCOMINGFILE_H
#define INCOMINGFILE_H

#include <QFile>
#include <QDir>
#include "config.h"
#include "core/settings.h"
#include "types/messagemedia.h"

class IncomingFile
{
public:
    enum IncomingFileType {
        UserThumbnail,
        UserPhoto,
        ChatThumbnail,
        ChatPhoto,
        MessagePhoto,
        MessageVideo,
        MessageActionPhoto,
        EncryptedMessagePhoto,
        EncryptedMessageVideo,
    };

    IncomingFile(const QString& downloadsPath);

    void setId(qint32 id);
    void setMediaId(qint64 mediaId);
    void setIncomingFileType(IncomingFileType type);
    void setMediaType(MessageMedia::MessageMediaType mediaType);
    void setFileLocationId(qint32 fileLocationId);
    void setSizeType(const QString &sizeType);
    void setIsThumbnail(bool isThumbnail);
    void setFilename(const QString &filename);
    void setModificationTime(qint32 mTime);
    void setExtension(const QString &extension);
    void setServerLocation(const FileLocation &serverLocation);
    void setExpectedSize(qint64 expectedSize);
    qint32 id() const;
    qint64 mediaId() const;
    IncomingFileType incomingFileType() const;
    MessageMedia::MessageMediaType mediaType() const;
    qint32 fileLocationId() const;
    QFile &file();
    FileLocation serverLocation() const;
    qint32 expectedSize() const;
    QString sizeType() const;
    bool isThumbnail() const;
    QString downloadsPath() const;
    QString filePath();


private:
    // depending on the file type, id will be a messageId, userId or chatId
    qint32 mId;
    qint64 mMediaId;
    IncomingFileType mIncomingFileType;
    MessageMedia::MessageMediaType mMediaType;
    qint32 mFileLocationId;
    QString mSizeType;
    bool mIsThumbnail;
    QString mFilename;
    QString mFilePath;
    qint32 mModificationTime;
    QString mExtension;
    QFile mFile;
    qint64 mExpectedSize;
    FileLocation mServerLocation;
    QString mDownloadsPath;
};

#endif // INCOMINGFILE_H
