/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "contactitem.h"

ContactItem::ContactItem(QObject *parent) :
    TLUser(parent) {
}

ContactItem::ContactItem(const ContactItem &other) {
    *this = other;
}

ContactItem::ContactItem(const User &u) {
    setId(u.id());
    setFirstName(u.firstName());
    setLastName(u.lastName());
    setPhone(u.phone());
    setStatus(u.status());
    setAccessHash(u.accessHash());
    setClassType(u.classType());
}

ContactItem::~ContactItem() {
}

ContactItem &ContactItem::operator=(const ContactItem &contactItem) {
    setId(contactItem.id());
    setFirstName(contactItem.firstName());
    setLastName(contactItem.lastName());
    setPhone(contactItem.phone());
    setThumbnail(contactItem.thumbnail());
    setBigPhoto(contactItem.bigPhoto());
    setStatus(*contactItem.status());
    setAccessHash(contactItem.accessHash());
    setClassType(contactItem.classType());
    return *this;
}

qint32 ContactItem::lastSeenOnline() const {
    return status()->wasOnline();
}

bool ContactItem::online() const {
    return status()->classType() == UserStatus::typeUserStatusOnline;
}

bool ContactItem::isEmpty() const  {
    return id() <= 0;
}

void ContactItem::dump() {
    qCDebug(TG_PLUGIN_LOGIC) << "UserID" << id();
    qCDebug(TG_PLUGIN_LOGIC) << "FirstName" << firstName();
    qCDebug(TG_PLUGIN_LOGIC) << "LatsName" << lastName();
    qCDebug(TG_PLUGIN_LOGIC) << "phone" << phone();
    qCDebug(TG_PLUGIN_LOGIC) << "thumbnail" << thumbnail();
    qCDebug(TG_PLUGIN_LOGIC) << "bigPhoto" << bigPhoto();
    qCDebug(TG_PLUGIN_LOGIC) << "lastSeenOnline" << lastSeenOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "online" << online();
    qCDebug(TG_PLUGIN_LOGIC) << "---";
}

