/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONTACTITEM_H
#define CONTACTITEM_H

#include "types/tluser.h"
#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(TG_PLUGIN_LOGIC)

class ContactItem : public TLUser {
    Q_OBJECT
    Q_PROPERTY(qint32 lastSeenOnline READ lastSeenOnline NOTIFY lastSeenOnlineChanged)
    Q_PROPERTY(bool online READ online NOTIFY onlineChanged)
    Q_PROPERTY(QString thumbnail READ thumbnail NOTIFY thumbnailChanged)
    Q_PROPERTY(QString bigPhoto READ bigPhoto WRITE setBigPhoto NOTIFY bigPhotoChanged)

public:

    explicit ContactItem(QObject *parent = 0);
    ContactItem(const ContactItem &other);
    ContactItem(const User &user);
    ~ContactItem();
    ContactItem &operator=(const ContactItem &);

    QString thumbnail() const { return m_thumbnail; }
    void setThumbnail(const QString &thumbnail) { m_thumbnail = thumbnail; }
    QString bigPhoto() const { return mBigPhoto; }
    void setBigPhoto(const QString &bigPhoto) { mBigPhoto = bigPhoto; }

    qint32 lastSeenOnline() const;
    bool online() const;

    bool isEmpty() const;
    void dump();

Q_SIGNALS:
    void lastSeenOnlineChanged(qint32);
    void onlineChanged(bool);
    void thumbnailChanged(QString);
    void bigPhotoChanged(QString);

private:
    QString m_thumbnail;
    QString mBigPhoto;
};
Q_DECLARE_METATYPE(ContactItem)


#endif // CONTACTITEM_H
