/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CONTACTSMODEL_H
#define CONTACTSMODEL_H

#include <QAbstractListModel>
#include "contactitem.h"
#include "contactsview.h"
#include "telegramclient.h"

class ContactsModel : public QAbstractListModel
{
    Q_OBJECT
    Q_ENUMS(ContactRoles)
public:

    enum ContactRoles {
        IdRole = Qt::UserRole,
        FirstNameRole,
        LastNameRole,
        PhoneRole,
        LastSeenOnlineRole,
        OnlineRole,
        ThumbnailRole
    };

    explicit ContactsModel(QObject *parent = 0);
    ~ContactsModel();

    // Overriding
    QHash<qint32, QByteArray> roleNames() const;
    qint32 rowCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    Q_INVOKABLE bool canFetchMore(const QModelIndex &parent = QModelIndex()) const;
    Q_INVOKABLE void fetchMore(const QModelIndex &parent = QModelIndex());

    Q_INVOKABLE void setTelegramClient(TelegramClient *telegramClient);

    void append(const ContactItem &contact);
    void clear();
    ContactItem &contact(qint32 id);
    void updateThumbnail(qint32 id, const QString &thumbnailPath);

public Q_SLOTS:
    Q_INVOKABLE void updateQuery();

private:
    QList<ContactItem> mContacts;
    // correlates contact->id() --> index in m_contacts
    QMap<qint32, qint32> mContactIdsMap;

    bool mCanFetchMore;

    // reference to telegram client
    TelegramClient *mTelegramClient;

private Q_SLOTS:
    void resetModel();

    void onNextPage(const QList<ContactItem> &contacts);

    void onContactsAdded(const QList<ContactItem> &contacts);
    void onContactThumbnailUpdated(qint32 id, const QString &thumbnail);
};

#endif // CONTACTSMODEL_H
