/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "contactsview.h"
#include "dbmanager.h"
#include <QDateTime>

ContactsView::ContactsView(const QSqlDatabase &database, QObject *parent) :
    View(database, parent) {
}

void ContactsView::getNextPage() {

    QList<ContactItem> contacts;

    mQuery.prepare(QString("SELECT id, firstName, lastName, phone, thumbnail, lastSeenOnline, online, type "
                           "FROM contactsView LIMIT %1 OFFSET %2")
                   .arg(QString::number(mPageSize), QString::number(mOffset)));
    if (!mQuery.exec()) {
        Q_EMIT invalidated();
        qCCritical(TG_PLUGIN_LOGIC) << "Error:" << mQuery.lastError() << mQuery.lastQuery();
    }

    while (mQuery.next()) {
        ContactItem contact;

        qint32 id = mQuery.value("id").toInt();
        QString firstName = mQuery.value("firstName").toString();
        QString lastName = mQuery.value("lastName").toString();
        QString phone = mQuery.value("phone").toString();
        QString thumbnail = mQuery.value("thumbnail").toString();
        qint32 lastSeenOnline = mQuery.value("lastSeenOnline").toInt();
        bool online = mQuery.value("online").toBool();
        qint32 type = mQuery.value("type").toInt();

        contact.setId(id);
        contact.setFirstName(firstName);
        contact.setLastName(lastName);
        contact.setPhone(phone);
        contact.setThumbnail(thumbnail);

        TLUserStatus status;
        status.setWasOnline(lastSeenOnline);
        online ? status.setClassType(UserStatus::typeUserStatusOnline) : status.setClassType(UserStatus::typeUserStatusOffline);
        contact.setClassType((User::UserType)type);

        contact.setStatus(status);

        contacts << contact;
    }

    mOffset += mPageSize;
    mQuery.clear();

    Q_EMIT nextPage(contacts);
}
