/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "dialogitem.h"
#include "data.h"

DialogItem::DialogItem(QObject *parent) :
    TLDialog(parent),
    m_id(0),
    m_isChat(0),
    m_name(""),
    mPeerId(0),
    mPeerType(InputPeer::typeInputPeerEmpty),
    m_thumbnail(""),
    mTyping(false),
    mWhoIsTyping(""),
    m_members(0),
    m_membersOnline(0),
    m_userOnline(false),
    m_lastSeenOnline(0),
    mDate(0),
    mIsSecret(false),
    mState(SecretChat::Init),
    mTimerId(0) {
}

DialogItem::DialogItem(const DialogItem &other) {
    *this = other;
}

DialogItem::~DialogItem() {
}

DialogItem &DialogItem::operator=(const DialogItem &dialogItem) {
    setId(dialogItem.id());
    setIsChat(dialogItem.isChat());
    setName(dialogItem.name());
    setPeerId(dialogItem.peerId());
    setUnreadCount(dialogItem.unreadCount());
    setThumbnail(dialogItem.thumbnail());
    setMembers(dialogItem.members());
    setMembersOnline(dialogItem.membersOnline());
    setUserOnline(dialogItem.userOnline());
    setLastSeenOnline(dialogItem.lastSeenOnline());
    setPeerType(dialogItem.peerType());
    setDate(dialogItem.date());
    setIsSecret(dialogItem.isSecret());
    setState(dialogItem.state());
    mTyping = dialogItem.mTyping;
    mWhoIsTyping = dialogItem.mWhoIsTyping;
    mTopMessage = dialogItem.mTopMessage;
    return *this;
}

//TODO Roberto: workaround sanity check method for avoid having unreadCount < 0
void DialogItem::setUnreadCount(qint32 unreadCount) {
    if (unreadCount < 0) {
        unreadCount = 0;
    }
    Dialog::setUnreadCount(unreadCount);
}

void DialogItem::setPeerType(qint32 peerType) {
    if (isChat()) {
        mPeerType = InputPeer::typeInputPeerChat;
    } else {
        switch (peerType) {
        case User::typeUserContact:
            mPeerType = InputPeer::typeInputPeerContact;
            break;
        case User::typeUserRequest:
        case User::typeUserForeign:
            mPeerType = InputPeer::typeInputPeerForeign;
            break;
        case User::typeUserDeleted:
        case User::typeUserEmpty:
            mPeerType = InputPeer::typeInputPeerEmpty;
            break;
        case User::typeUserSelf:
            mPeerType = InputPeer::typeInputPeerSelf;
            break;
        default:
            mPeerType = peerType;
        }
    }
}

void DialogItem::setTyping(bool typing) {
    mTyping = typing;

    //remove previous timer if alive
    if (mTimerId != 0) {
        killTimer(mTimerId);
        mTimerId = 0;
    }

    if (mTyping) {
        mTimerId = startTimer(TYPING_INTERVAL);
    } else {
        setWhoIsTyping("");
        Q_EMIT typingFinished();
    }
}

void DialogItem::timerEvent(QTimerEvent *timerEvent) {
    if (mTimerId == timerEvent->timerId()) {
        killTimer(mTimerId);
        mTimerId = 0;
        mTyping = false;
        mWhoIsTyping = "";
        Q_EMIT typingFinished();
    }
}

bool DialogItem::isEmpty() const {
    return id() <= 0;
}

void DialogItem::dump() {
    qCDebug(TG_PLUGIN_LOGIC) << "ID" << id();
    qCDebug(TG_PLUGIN_LOGIC) << "ischat" << isChat();
    qCDebug(TG_PLUGIN_LOGIC) << "name" << name();
    qCDebug(TG_PLUGIN_LOGIC) << "peerId" << peerId();
    qCDebug(TG_PLUGIN_LOGIC) << "peerType" << peerType();
    qCDebug(TG_PLUGIN_LOGIC) << "unreadCount" << unreadCount();
    qCDebug(TG_PLUGIN_LOGIC) << "thumbnail" << thumbnail();
    qCDebug(TG_PLUGIN_LOGIC) << "typing" << typing();
    qCDebug(TG_PLUGIN_LOGIC) << "whoisTyping" << whoisTyping();
    qCDebug(TG_PLUGIN_LOGIC) << "members" << members();
    qCDebug(TG_PLUGIN_LOGIC) << "membersOnline" << membersOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "useronline" << userOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "lastSeenOnline" << lastSeenOnline();
    qCDebug(TG_PLUGIN_LOGIC) << "date" << date();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageId" << topMessage().id();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromId" << topMessage().fromId();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromFirstName" << topMessage().fromFirstName();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromLastName" << topMessage().fromLastName();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageFromPhone" << topMessage().fromPhone();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageOut" << topMessage().out();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageUnread" << topMessage().unread();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageText" << topMessage().text();
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageMediaType" << QString::number(topMessage().mediaType());
    qCDebug(TG_PLUGIN_LOGIC) << "topMessageDate" << topMessage().date();
    qCDebug(TG_PLUGIN_LOGIC) << "---";
}
