/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DIALOGITEM_H
#define DIALOGITEM_H

#define TYPING_INTERVAL 5000

#include "types/tldialog.h"
#include "messageitem.h"
#include <QTimerEvent>
#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(TG_PLUGIN_LOGIC)

class DialogItem : public TLDialog {
    Q_OBJECT
    //FIXME I thinks these Q_PROPERTY are not needed
    Q_PROPERTY(qint32 id READ id)
    Q_PROPERTY(bool isChat READ isChat)
    Q_PROPERTY(QString name READ name)
    Q_PROPERTY(QString thumbnail READ thumbnail)
    Q_PROPERTY(bool typing READ typing)
    Q_PROPERTY(QString whoisTyping READ whoisTyping)
    Q_PROPERTY(qint32 members READ members)
    Q_PROPERTY(qint32 membersOnline READ membersOnline)
    Q_PROPERTY(bool userOnline READ userOnline)
    Q_PROPERTY(qint32 lastSeenOnline READ lastSeenOnline)
    Q_PROPERTY(qint32 peerId READ peerId)
    Q_PROPERTY(qint32 peerType READ peerType)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(bool isSecret READ isSecret)
    Q_PROPERTY(qint32 state READ state)

public:

    explicit DialogItem(QObject *parent = 0);
    DialogItem(const DialogItem &other);
    DialogItem &operator=(const DialogItem &);
    ~DialogItem();

    qint32 id() const { return m_id; }
    bool isChat() const { return m_isChat; }
    QString name() const { return m_name; }
    QString thumbnail() const { return m_thumbnail; }
    bool typing() const { return mTyping; }
    QString whoisTyping() const { return mWhoIsTyping; }
    qint32 members() const { return m_members; }
    qint32 membersOnline() const { return m_membersOnline; }
    bool userOnline() const { return m_userOnline; }
    qint32 lastSeenOnline() const { return m_lastSeenOnline; }
    qint32 peerId() const { return mPeerId; }
    qint32 peerType() const { return mPeerType; }
    qint32 date() const { return mDate; }
    bool isSecret() const { return mIsSecret; }
    qint32 state() const { return mState; }
    MessageItem &topMessage() { return mTopMessage; }

    void setId(qint32 id) { m_id = id; }
    void setIsChat(bool isChat) { m_isChat = isChat; }
    void setName(const QString &name) { m_name = name; }
    void setThumbnail(QString thumbnail) { m_thumbnail = thumbnail; }
    void setMembers(qint32 members) { m_members = members; }
    void setMembersOnline(qint32 membersOnline) { m_membersOnline = membersOnline; }
    void setUserOnline(bool userOnline) { m_userOnline = userOnline; }
    void setLastSeenOnline(qint32 lastSeenOnline) { m_lastSeenOnline = lastSeenOnline; }
    void setPeerId(qint32 peerId) { mPeerId = peerId; }
    void setPeerType(qint32 peerType);
    void setDate(qint32 date) { mDate = date; }
    void setIsSecret(bool isSecret) { mIsSecret = isSecret; }
    void setState(qint32 state) { mState = state; }
    void setTopMessage(const MessageItem &message) { mTopMessage = message; }
    void setTyping(bool typing);
    void setWhoIsTyping(const QString &whoisTyping) { mWhoIsTyping = whoisTyping; }

    //TODO Roberto: workaround sanity check method for avoid having unreadCount < 0
    void setUnreadCount(qint32 unreadCount);

    bool isEmpty() const;
    void dump();

Q_SIGNALS:
    void typingFinished();

private:
    // dialog identifier
    qint32 m_id;
    // flag to set if this dialog belongs to a chat. if false, is a user
    bool m_isChat;
    // name of the user / title of the chat
    QString m_name;
    qint32 mPeerId;
    qint32 mPeerType;
    QString m_thumbnail;
    bool mTyping;
    QString mWhoIsTyping; // full name of the peer typing in a group chat. Empty if the dialog is a secret chat or a 1-1 chat
    qint32 m_members; //XXX Could this be referenced to a chats list in Cache?
    qint32 m_membersOnline;
    bool m_userOnline;
    qint32 m_lastSeenOnline;
    qint32 mDate; // This is the last added message date. In case of secret chat without messages, the date of chat creation
    bool mIsSecret;
    qint32 mState;
    MessageItem mTopMessage;

    qint32 mTimerId;

    void timerEvent(QTimerEvent *timerEvent);
};
Q_DECLARE_METATYPE(DialogItem)

#endif // DIALOGITEM_H
