/*
 * Copyright 2015 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DIALOG_MODEL_H
#define DIALOG_MODEL_H

#include <QObject>
#include <QSharedPointer>
#include <QString>
#include <QTimerEvent>

#include "telegramclient.h"
#include "types/types.h"
#include "dbmanager.h"

class DialogModel : public QObject
{
    Q_OBJECT
    Q_PROPERTY(qint32 chatId READ getChatId NOTIFY chatIdChanged)
    Q_PROPERTY(qint32 peerId READ getPeerId NOTIFY peerIdChanged)
    Q_PROPERTY(qint32 peerType READ getPeerType NOTIFY peerTypeChanged)
    Q_PROPERTY(bool isChat READ getIsChat NOTIFY isChatChanged)
    Q_PROPERTY(QString whoIsTyping READ getWhoIsTyping NOTIFY whoIsTypingChanged)
    Q_PROPERTY(bool isTyping READ getIsTyping NOTIFY isTypingChanged)
    Q_PROPERTY(bool isSecret READ getIsSecret NOTIFY isSecretChanged)
    Q_PROPERTY(qint32 state READ getState NOTIFY stateChanged)
    Q_PROPERTY(QString title READ getTitle NOTIFY titleChanged)
    Q_PROPERTY(QString phone READ getPhone NOTIFY phoneChanged)
    Q_PROPERTY(qint32 ttl READ getTtl NOTIFY ttlChanged)

public:
    explicit DialogModel(QObject *parent = 0);
    ~DialogModel();

    Q_INVOKABLE void setup(TelegramClient *telegramClient, qint32 chatId);
    void resetDialogDetails();

    qint32 getChatId() const;
    void setChatId(qint32 chatId);

    qint32 getPeerId() const;
    void setPeerId(qint32 peerId);

    qint32 getPeerType() const;
    void setPeerType(qint32 classType);

    bool getIsChat() const;
    void setIsChat(bool isChat);

    QString getWhoIsTyping() const;
    void setWhoIsTyping(QString whoIsTyping);

    bool getIsTyping() const;
    void setIsTyping(bool isTyping);

    bool getIsSecret() const;
    void setIsSecret(bool isSecret);

    qint32 getState() const;
    void setState(qint32 state);

    QString getTitle() const;
    void setTitle(const QString &title);

    QString getPhone() const;
    void setPhone(QString phone);

    qint32 getTtl() const;

public Q_SLOTS:
    void refreshDialogDetails(qint32 chatId);
    void userTyping(qint32 chatId, QString who);
    void setTtl(qint32 ttl);
    void onMessageReceived(const MessageItem &message);

Q_SIGNALS:
    void dialogDetailsRefreshed();

private:

Q_SIGNALS:
    void chatIdChanged(qint32 chatId);
    void peerIdChanged(qint32 peerId);
    void peerTypeChanged(qint32 peerType);
    void peerChanged(qint32 peerId, qint32 peerType);
    void isChatChanged(bool isChat);
    void whoIsTypingChanged(QString who);
    void isTypingChanged(bool isTyping);
    void isSecretChanged(bool isSecret);
    void stateChanged(qint32 state);
    void titleChanged(QString title);
    void phoneChanged(QString phone);
    void ttlChanged(qint32 ttl);

private:
    TelegramClient *mTelegramClient;

    qint32 mChatId;
    qint32 mPeerId;
    qint32 mPeerType;
    bool mIsChat;
    bool mIsTyping;
    QString mWhoIsTyping;
    bool mIsSecret;
    qint32 mState;
    QString mTitle;
    QString mPhone;
    qint32 mTtl;

    qint32 mTypingTimerId;
    void timerEvent(QTimerEvent *timerEvent);
};

#endif
