/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "dialogsproxy.h"
#include "dialogsmodel.h"
#include "core/settings.h"
#include "telegramclient.h"

DialogsProxy::DialogsProxy(QObject *parent) :
    SortProxyModel(parent) {
}

QVariant DialogsProxy::get(qint32 row) {
    QModelIndex proxyIndex = index(row, 0);
    QModelIndex modelIndex = mapToSource(proxyIndex);
    return static_cast<DialogsModel *>(sourceModel())->get(modelIndex.row());
}

QVariant DialogsProxy::getDialogFromDialogId(qint32 dialogId) {
    return static_cast<DialogsModel *>(sourceModel())->getDialogFromDialogId(dialogId);
}

bool DialogsProxy::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const {
    QModelIndex index = sourceModel()->index(sourceRow, 0, sourceParent);

    QString name = sourceModel()->data(index, DialogsModel::NameRole).toString();
    qint32 id = sourceModel()->data(index, DialogsModel::IdRole).toInt();

    bool notSelf = id != Settings::getInstance()->ourId();
    return notSelf && name.contains(filterRegExp());
}

//void DialogsProxy::setTelegramClient(QObject *tc) {
//    DialogsModel *dialogsModel = static_cast<DialogsModel *>(sourceModel());
//    TelegramClient *telegramClient = static_cast<TelegramClient *>(tc);
//    dialogsModel->setTelegramClient(telegramClient);
//}
