/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "dialogsview.h"
#include "dbmanager.h"
#include <QDateTime>

DialogsView::DialogsView(const QSqlDatabase &database, QObject *parent) :
    View(database, parent) {
}

void DialogsView::getNextPage() {

    QList<DialogItem> dialogs;

    mQuery.prepare(QString("SELECT id, isChat, name, unreadCount, thumbnail, members, userOnline, lastSeenOnline, peerId, peerType, date, "
                           "isSecret, state, topMessageId, topMessageFromId, topMessageFromFirstName, topMessageFromLastName, "
                           "topMessageFromPhone, topMessageToId, topMessageOut, topMessageUnread, topMessageSent, topMessageDate, "
                           "topMessageText, topMessageMediaType, topMessageActionType, topMessageActionTitle, topMessageActionUser "
                           "FROM dialogsView LIMIT %1 OFFSET %2").arg(QString::number(mPageSize), QString::number(mOffset)));
    if (!mQuery.exec()) {
        Q_EMIT invalidated();
        qCCritical(TG_PLUGIN_LOGIC) << "Error:" << mQuery.lastError() << mQuery.lastQuery();
    }


    while (mQuery.next()) {
        DialogItem dialog;

        qint32 id = mQuery.value("id").toInt();
        bool isChat = mQuery.value("isChat").toBool();
        QString name = mQuery.value("name").toString();
        qint32 unreadCount = mQuery.value("unreadCount").toInt();
        QString thumbnail = mQuery.value("thumbnail").toString();
        qint32 members = mQuery.value("members").toInt();
        bool userOnline = mQuery.value("userOnline").toBool();
        qint32 lastSeenOnline = mQuery.value("lastSeenOnline").toInt();
        qint32 peerId = mQuery.value("peerId").toInt();
        qint32 peerType = mQuery.value("peerType").toInt();
        qint32 date = mQuery.value("date").toInt();
        bool isSecret = mQuery.value("isSecret").toBool();
        qint32 state = mQuery.value("state").toInt();
        qint32 topMessageId = mQuery.value("topMessageId").toInt();
        qint32 topMessageFromId = mQuery.value("topMessageFromId").toInt();
        QString topMessageFromFirstName = mQuery.value("topMessageFromFirstName").toString();
        QString topMessageFromLastName = mQuery.value("topMessageFromLastName").toString();
        QString topMessageFromPhone = mQuery.value("topMessageFromPhone").toString();
        qint32 topMessageToId = mQuery.value("topMessageToId").toInt();
        bool topMessageOut = mQuery.value("topMessageOut").toBool();
        qint32 topMessageUnread = mQuery.value("topMessageUnread").toInt();
        bool topMessageSent = mQuery.value("topMessageSent").toBool();
        qint32 topMessageDate = mQuery.value("topMessageDate").toInt();
        QString topMessageText = mQuery.value("topMessageText").toString();
        qint32 topMessageMediaType = mQuery.value("topMessageMediaType").toInt();
        qint32 topMessageActionType = mQuery.value("topMessageActionType").toInt();
        QString topMessageActionTitle = mQuery.value("topMessageActionTitle").toString();
        QString topMessageActionUser = mQuery.value("topMessageActionUser").toString();

        dialog.setId(id);
        dialog.setIsChat(isChat);
        dialog.setName(name);
        dialog.setUnreadCount(unreadCount);
        dialog.setThumbnail(thumbnail);
        dialog.setMembers(members);
        dialog.setUserOnline(userOnline);
        dialog.setLastSeenOnline(lastSeenOnline);
        dialog.setDate(date);
        dialog.setIsSecret(isSecret);
        dialog.setState(state);
        peerId ? dialog.setPeerId(peerId) : dialog.setPeerId(id);
        isChat ? dialog.setPeerType(InputPeer::typeInputPeerChat) : dialog.setPeerType(peerType);

        MessageItem message;
        message.setId(topMessageId);
        message.setFromId(topMessageFromId);
        message.setFromFirstName(topMessageFromFirstName);
        message.setFromLastName(topMessageFromLastName);
        message.setFromPhone(topMessageFromPhone);
        Peer peer(Peer::typePeerUser);
        isChat ? peer.setChatId(topMessageToId) : peer.setUserId(topMessageToId);
        message.setToId(peer);
        message.setOut(topMessageOut);
        message.setUnread(topMessageUnread);
        message.setSent(topMessageSent);
        message.setDate(topMessageDate);
        message.setMessage(topMessageText);
        MessageMedia media((MessageMedia::MessageMediaType)topMessageMediaType);
        message.setMedia(media);

        if (topMessageActionType) {
            MessageAction action((MessageAction::MessageActionType)topMessageActionType);
            action.setTitle(topMessageActionTitle);
            message.setAction(action);
            message.setActionUser(topMessageActionUser);
        }

        dialog.setTopMessage(message);

        dialogs << dialog;
    }

    mOffset += mPageSize;
    mQuery.clear();

    Q_EMIT nextPage(dialogs);
}
