/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GROUP_MEMBERS_MODEL_H
#define GROUP_MEMBERS_MODEL_H

#include <QLoggingCategory>

#include "model.h"
#include "telegramclient.h"

class GroupMembersModel : public Model
{
    Q_OBJECT
    Q_ENUMS(GroupMemberRoles)
public:

    enum GroupMemberRoles {
        IdRole = Qt::UserRole,
        NameRole,
        PhotoRole,
        LastSeenOnlineRole,
        OnlineRole,
        IsAdminRole,
        IsRemovableRole,
        PeerTypeRole
    };

    explicit GroupMembersModel(QObject *parent = 0);
    ~GroupMembersModel();

    Q_INVOKABLE void setup(TelegramClient *telegramClient, qint32 chatId);
    Q_INVOKABLE qint32 getInputPeerType(qint32 userType);

public Q_SLOTS:
    void refresh(qint32 chatId);

private:
    TelegramClient *mTelegramClient;
    qint32 mChatId;

    QHash<qint32, QByteArray> roleNames() const;
};

#endif // GROUP_MEMBERS_MODEL_H
