/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GROUP_MODEL_H
#define GROUP_MODEL_H

#include <QObject>
#include <QString>

#include "telegramclient.h"
#include "dbmanager.h"
#include "groupmembersmodel.h"

class GroupModel : public QObject
{
    Q_OBJECT
    Q_PROPERTY(qint32 chatId READ getChatId WRITE setChatId NOTIFY chatIdChanged)
    Q_PROPERTY(QString title READ getTitle NOTIFY titleChanged)
    Q_PROPERTY(QString thumbnail READ getThumbnail NOTIFY thumbnailChanged)
    Q_PROPERTY(QString photo READ getPhoto NOTIFY photoChanged)
    Q_PROPERTY(qint32 count READ getCount NOTIFY countChanged)
    Q_PROPERTY(bool muted READ getMuted NOTIFY mutedChanged)

public:
    explicit GroupModel(QObject *parent = 0);
    ~GroupModel();

    Q_INVOKABLE void setup(TelegramClient *telegramClient, qint32 chatId);
    void resetGroupDetails();

    void setChatId(qint32 chatId);
    qint32 getChatId() const;
    QString getTitle() const;
    QString getThumbnail() const;
    QString getPhoto() const;
    qint32 getCount() const;
    bool getMuted() const;

public Q_SLOTS:
    void refreshGroupDetails(qint32 chatId);
    void chatThumbnailUpdated(qint32 chatId, QString thumbnail);
    void chatMuted(qint32 chatId, bool muted);
    void chatsMuted(NotifyPeer::NotifyPeerType notifyType, bool muted);

Q_SIGNALS:
    void groupDetailsRefreshed();

private:
    void setTitle(const QString &title);
    void setThumbnail(const QString &thumbnail);
    void setPhoto(const QString &photo);
    void setCount(const qint32 count);
    void setMuted(bool muted);

Q_SIGNALS:
    void chatIdChanged(qint32 chatId);
    void titleChanged(QString title);
    void thumbnailChanged(QString thumbnail);
    void photoChanged(QString photo);
    void countChanged(qint32 count);
    void mutedChanged(bool muted);

private:
    TelegramClient* mTelegramClient;

    qint32 mChatId;
    QString mTitle;
    QString mThumbnail;
    QString mPhoto;
    qint32 mCount;
    bool mMuted;
};

#endif
