/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MESSAGEITEM_H
#define MESSAGEITEM_H

#include "types/tlmessage.h"
#include "types/tlmessagemedia.h"
#include "types/tlpeer.h"
#include <types/user.h>
#include "models/contactitem.h"

class MessageItem : public TLMessage {
    Q_OBJECT
    Q_PROPERTY(QString dialogId READ dialogId NOTIFY dialogIdChanged)
    Q_PROPERTY(QString fromFirstName READ fromFirstName NOTIFY fromFirstNameChanged)
    Q_PROPERTY(QString fromLastName READ fromLastName NOTIFY fromLastNameChanged)
    Q_PROPERTY(QString fromPhone READ fromPhone NOTIFY fromPhoneChanged)
    Q_PROPERTY(QString fromThumbnail READ fromThumbnail NOTIFY fromThumbnailChanged)
    Q_PROPERTY(qint32 fromUserType READ fromUserType NOTIFY fromUserTypeChanged)
    Q_PROPERTY(qint32 toId READ toId NOTIFY toIdChanged)
    Q_PROPERTY(bool toIsChat READ toIsChat NOTIFY toIsChatChanged)
    Q_PROPERTY(qint32 ttl READ ttl NOTIFY ttlChanged)
    Q_PROPERTY(QString text READ text NOTIFY textChanged)
    Q_PROPERTY(qint32 mediaType READ mediaType NOTIFY mediaTypeChanged)
    Q_PROPERTY(QString thumbnail READ thumbnail NOTIFY thumbnailChanged)
    Q_PROPERTY(QString photo READ photo NOTIFY photoChanged)
    Q_PROPERTY(QString video READ video NOTIFY videoChanged)
    Q_PROPERTY(bool downloading READ downloading WRITE setDownloading NOTIFY downloadingChanged)
    Q_PROPERTY(qint8 downloadedPercentage READ downloadedPercentage NOTIFY downloadedPercentageChanged)
    Q_PROPERTY(bool isAction READ isAction NOTIFY isActionChanged)
    Q_PROPERTY(bool sent READ sent NOTIFY sentChanged)
    Q_PROPERTY(qint32 actionType READ actionType NOTIFY actionTypeChanged)
    Q_PROPERTY(QString actionTitle READ actionTitle NOTIFY actionTitleChanged)
    Q_PROPERTY(QString actionUser READ actionUser NOTIFY actionUserChanged)
public:

    explicit MessageItem(QObject *parent = 0);
    MessageItem(const MessageItem &other);
    MessageItem &operator=(const MessageItem &);
    bool operator==(const MessageItem &) const;
    MessageItem(const Message &other, const ContactItem &sender, bool sent = true);
    ~MessageItem();

    qint32 dialogId() const;
    QString fromFirstName() const;
    QString fromLastName() const;
    QString fromPhone() const;
    QString fromThumbnail() const;
    qint32 fromUserType() const;
    qint32 toId() const;
    bool toIsChat() const;
    QString text() const;
    TLMessageMedia::MessageMediaType mediaType() const;
    QString thumbnail() const;
    QString photo() const;
    QString video() const;
    bool downloading() const;
    qint8 downloadedPercentage() const;
    QString photoSizeType() const;
    bool isAction() const;
    bool sent() const;
    qint32 actionType() const;
    QString actionTitle() const;
    QString actionUser() const;

    void setDialogId(qint32 dialogId);
    void setThumbnail(const QString &thumbnail);
    void setPhoto(const QString &photo);
    void setPhotoSizeType(const QString &photoSizeType);
    void setVideo(const QString &video);
    void setDownloading(bool downloading);
    void setDownloadedPercentage(qint8 downloadedPercentage);
    void setFromFirstName(const QString &firstName);
    void setFromLastName(const QString &lastName);
    void setFromPhone(const QString &phone);
    void setFromThumbnail(const QString &fromThumb);
    void setFromUserType(qint32 fromUserType);
    void setSender(const ContactItem &sender);
    void setSent(bool sent);
    void setActionUser(const QString &actionUser);

    bool isEmpty() const;

    void dump();

Q_SIGNALS:
    void dialogIdChanged(qint32);
    void fromFirstNameChanged(QString);
    void fromLastNameChanged(QString);
    void fromPhoneChanged(QString);
    void fromThumbnailChanged(QString);
    void fromUserTypeChanged(qint32);
    void toIdChanged(qint32);
    void toIsChatChanged(bool);
    void ttlChanged(qint32);
    void textChanged(QString);
    void mediaTypeChanged(qint32);
    void thumbnailChanged(QString);
    void photoChanged(QString);
    void videoChanged(QString);
    void downloadingChanged(bool);
    void downloadedPercentageChanged(qint8);
    void isActionChanged(bool);
    void sentChanged(bool);
    void actionTypeChanged(qint32);
    void actionTitleChanged(QString);
    void actionUserChanged(QString);

private:
    qint32 mDialogId;
    QString mThumbnail;
    QString mPhoto;
    QString mPhotoSizeType;
    QString mVideo;
    bool mDownloading;
    qint8 mDownloadedPercentage;
    QString mFromFirstName;
    QString mFromLastName;
    QString mFromPhone;
    QString mFromThumbnail;
    qint32 mFromUserType;
    bool mSent;
    QString mActionUser;
};
Q_DECLARE_METATYPE(MessageItem)

#endif // MESSAGEITEM_H
