/*
 * Copyright 2014 Canonical, Ltd.
 *
 * Authors:
 *  Tiago Salem Herrmann <tiago.herrmann@canonical.com>
 *
 * This file is part of history-service.
 *
 * history-service is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * history-service is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "sortproxymodel.h"
#include <QDebug>

SortProxyModel::SortProxyModel(QObject *parent) :
    QSortFilterProxyModel(parent), mAscending(true)
{
    setDynamicSortFilter(true);
    updateSorting();
}

bool SortProxyModel::ascending() const
{
    return mAscending;
}

void SortProxyModel::setAscending(bool value)
{
    if (mAscending != value) {
        mAscending = value;
        updateSorting();
        Q_EMIT ascendingChanged();
    }
}

QString SortProxyModel::searchTerm() const {
    return mSearchTerm;
}

void SortProxyModel::setSearchTerm(QString searchTerm) {
    if (mSearchTerm != searchTerm) {
        mSearchTerm = searchTerm;

        QString pattern = QString("\\b%1").arg(searchTerm);
        setFilterKeyColumn(0);
        setFilterRegExp(QRegExp(pattern, Qt::CaseInsensitive));

        Q_EMIT searchTermChanged();
    }
}

QVariant SortProxyModel::get(int rowIndex) const {
    QVariantMap data;
    QModelIndex sourceIndex = mapToSource(index(rowIndex, 0));
    if (sourceIndex.isValid()) {
        QAbstractItemModel *source = sourceModel();
        QHash<int, QByteArray> roles = source->roleNames();
        Q_FOREACH(int role, roles.keys()) {
            data.insert(roles[role], source->data(sourceIndex, role));
        }
    }
    return data;
}

void SortProxyModel::updateSorting()
{
    sort(0, mAscending ? Qt::AscendingOrder : Qt::DescendingOrder);
}
