/*
 * Copyright 2014 Canonical, Ltd.
 *
 * Authors:
 *  Tiago Salem Herrmann <tiago.herrmann@canonical.com>
 *
 * This file is part of history-service.
 *
 * history-service is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * history-service is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SORTPROXYMODEL_H
#define SORTPROXYMODEL_H

#include <QSortFilterProxyModel>

class SortProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
    Q_PROPERTY(bool ascending READ ascending WRITE setAscending NOTIFY ascendingChanged)
    Q_PROPERTY(int count READ rowCount NOTIFY countChanged)
    Q_PROPERTY(QString searchTerm READ searchTerm WRITE setSearchTerm NOTIFY searchTermChanged)

public:
    explicit SortProxyModel(QObject *parent = 0);

    bool ascending() const;
    void setAscending(bool value);
    Q_INVOKABLE QVariant get(int rowIndex) const;

    QString searchTerm() const;
    void setSearchTerm(QString searchTerm);

private Q_SLOTS:
    void updateSorting();

Q_SIGNALS:
    void ascendingChanged();
    void searchTermChanged();
    void countChanged();

private:
    bool mAscending;
    QString mSearchTerm;
};

#endif // SORTPROXYMODEL_H
