/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "rawapiclient.h"
#include <QMetaType>

RawApiClient::RawApiClient(QObject *parent) :
    QObject(parent) {
    //output types
    qRegisterMetaType<TLAudio>("TLAudio");
    qRegisterMetaType<TLChat>("TLChat");
    qRegisterMetaType<TLChatParticipant>("TLChatParticipant");
    qRegisterMetaType<TLChatParticipants>("TLChatParticipants");
    qRegisterMetaType<TLChatPhoto>("TLChatPhoto");
    qRegisterMetaType<TLContact>("TLContact");
    qRegisterMetaType<TLContactBlocked>("TLContactBlocked");
    qRegisterMetaType<TLContactsForeignLink>("TLContactsForeignLink");
    qRegisterMetaType<TLContactsLink>("TLContactsLink");
    qRegisterMetaType<TLContactsMyLink>("TLContactsMyLink");
    qRegisterMetaType<TLContactStatus>("TLContactStatus");
    qRegisterMetaType<TLDcOption>("TLDcOption");
    qRegisterMetaType<TLDialog>("TLDialog");
    qRegisterMetaType<TLDocument>("TLDocument");
    qRegisterMetaType<TLEncryptedChat>("TLEncryptedChat");
    qRegisterMetaType<TLEncryptedFile>("TLEncryptedFile");
    qRegisterMetaType<TLEncryptedMessage>("TLEncryptedMessage");
    qRegisterMetaType<TLFileLocation>("TLFileLocation");
    qRegisterMetaType<TLGeoChatMessage>("TLGeoChatMessage");
    qRegisterMetaType<TLGeoPoint>("TLGeoPoint");
    qRegisterMetaType<TLImportedContact>("TLImportedContact");
    qRegisterMetaType<TLMessage>("TLMessage");
    qRegisterMetaType<TLMessageAction>("TLMessageAction");
    qRegisterMetaType<TLMessageMedia>("TLMessageMedia");
    qRegisterMetaType<TLMessagesFilter>("TLMessagesFilter");
    qRegisterMetaType<TLNotifyPeer>("TLNotifyPeer");
    qRegisterMetaType<TLPeer>("TLPeer");
    qRegisterMetaType<TLPeerNotifySettings>("TLPeerNotifySettings");
    qRegisterMetaType<TLPhoto>("TLPhoto");
    qRegisterMetaType<TLPhotoSize>("TLPhotoSize");
    qRegisterMetaType<TLStorageFileType>("TLStorageFileType");
    qRegisterMetaType<TLUpdate>("TLUpdate");
    qRegisterMetaType<TLUser>("TLUser");
    qRegisterMetaType<TLUserProfilePhoto>("TLUserProfilePhoto");
    qRegisterMetaType<TLUserStatus>("TLUserStatus");
    qRegisterMetaType<TLVideo>("TLVideo");
    //input types
    qRegisterMetaType<TLInputContact>("TLInputContact");
    qRegisterMetaType<TLInputFileLocation>("TLInputFileLocation");
    qRegisterMetaType<TLInputGeoPoint>("TLInputGeoPoint");
    qRegisterMetaType<TLInputPeer>("TLInputPeer");
    qRegisterMetaType<TLInputUser>("TLInputUser");
    qRegisterMetaType<TLMessagesFilter>("TLMessagesFilter");

    // register types for having queued connection in signal-slots
    qRegisterMetaType<QList<qint64> >("QList<qint64>");
    qRegisterMetaType<QList<qint32> >("QList<qint32>");
    qRegisterMetaType<QList<Message> >("QList<Message>");
    qRegisterMetaType<QList<EncryptedMessage> >("QList<EncryptedMessage>");
    qRegisterMetaType<QList<Dialog> >("QList<Dialog>");
    qRegisterMetaType<QList<Chat> >("QList<Chat>");
    qRegisterMetaType<QList<User> >("QList<User>");
    qRegisterMetaType<QList<Update> >("QList<Update>");
    qRegisterMetaType<QList<ContactsLink> >("QList<ContactsLink>");
    qRegisterMetaType<QList<InputUser> >("QList<InputUser>");
    qRegisterMetaType<QList<InputContact> >("QList<InputContact>");
    qRegisterMetaType<QList<ImportedContact> >("QList<ImportedContact>");
    qRegisterMetaType<QList<Photo> >("QList<Photo>");
    qRegisterMetaType<QList<ContactStatus> >("QList<ContactStatus>");
    qRegisterMetaType<QList<Contact> >("QList<Contact>");
    qRegisterMetaType<QList<ImportedContact> >("QList<ImportedContact>");
    qRegisterMetaType<QList<ContactBlocked> >("QList<ContactBlocked>");
    qRegisterMetaType<ContactsLink>("ContactsLink");
    qRegisterMetaType<ContactsMyLink>("ContactsMyLink");
    qRegisterMetaType<ContactsForeignLink>("ContactsForeignLink");
    qRegisterMetaType<Photo>("Photo");
    qRegisterMetaType<Message>("Message");
    qRegisterMetaType<User>("User");
    qRegisterMetaType<Update>("Update");
    qRegisterMetaType<UpdatesState>("UpdatesState");
    qRegisterMetaType<StorageFileType>("StorageFileType");
    qRegisterMetaType<PeerNotifySettings>("PeerNotifySettings");
    qRegisterMetaType<NotifyPeer::NotifyPeerType>("NotifyPeer::NotifyPeerType");

    // create service, thread, and initialization
    mTelegramService = new TelegramService;
    mServiceThread = new ServiceThread;
    mTelegramService->moveToThread(mServiceThread);
    // connect finish signals for exitting thread
    connect(mTelegramService, SIGNAL(finished()), mServiceThread, SLOT(quit()));
    connect(mTelegramService, SIGNAL(finished()), mTelegramService, SLOT(deleteLater()));
    connect(mServiceThread, SIGNAL(finished()), mServiceThread, SLOT(deleteLater()));

    // raw api signals management
    // Errors signal-slot
    connect(mTelegramService, SIGNAL(fatalError()), this, SIGNAL(fatalError()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(error(qint64,qint32,QString)), this, SIGNAL(error(qint64,qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSignInError(qint64,qint32,QString)), this, SIGNAL(authSignInError(qint64,qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSignUpError(qint64,qint32,QString)), this, SIGNAL(authSignUpError(qint64,qint32,QString)), Qt::QueuedConnection);
    // Registration / authorization signal-slot
    connect(mTelegramService, SIGNAL(authNeeded()), this, SIGNAL(authNeeded()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authLoggedIn()), this, SIGNAL(authLoggedIn()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), this, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authCheckPhoneSent(qint64, QString)), this, SIGNAL(authCheckPhoneSent(qint64, QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), this, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSendCallAnswer(qint64,bool)), this, SIGNAL(authSendCallAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authLogOutAnswer(qint64,bool)), this, SIGNAL(authLogOutAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSendInvitesAnswer(qint64,bool)), this, SIGNAL(authSendInvitesAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)), this, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)), Qt::QueuedConnection);
    // Working with notifications and settings
    connect(mTelegramService, SIGNAL(accountRegisterDeviceAnswer(qint64,bool)), this, SIGNAL(accountRegisterDeviceAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(accountUnregisterDeviceAnswer(qint64,bool)), this, SIGNAL(accountUnregisterDeviceAnswer(qint64,bool)), Qt::QueuedConnection);
    // Working with users signal-slot
    connect(mTelegramService, SIGNAL(usersGetUsersAnswer(qint64,QList<User>)), this, SLOT(onUsersGetUsersAnswer(qint64,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(usersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)), this, SLOT(onUsersGetFullUserAnswer(qint64,User,ContactsLink,Photo,PeerNotifySettings,bool,QString,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(photosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)), this, SLOT(onPhotosGetUserPhotosAnswer(qint64,qint32,QList<Photo>,QList<User>)), Qt::QueuedConnection);
    // Working with contacts signal-slot
    connect(mTelegramService, SIGNAL(contactsGetStatusesAnswer(qint64,QList<ContactStatus>)), this, SLOT(onContactsGetStatusesAnswer(qint64,QList<ContactStatus>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)), this, SLOT(onContactsGetContactsAnswer(qint64,bool,QList<Contact>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)), this, SLOT(onContactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)), this, SLOT(onContactsDeleteContactAnswer(qint64,ContactsMyLink,ContactsForeignLink,User)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsDeleteContactsAnswer(qint64,bool)), this, SIGNAL(contactsDeleteContactsAnswer(qint64,bool)), Qt::QueuedConnection);
    // Working with blacklist signal-slot
    connect(mTelegramService, SIGNAL(contactsBlockAnswer(qint64,bool)), this, SIGNAL(contactsBlockAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsUnblockAnswer(qint64,bool)), this, SIGNAL(contactsUnblockAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)), this, SLOT(onContactsGetBlockedAnswer(qint64,qint32,QList<ContactBlocked>,QList<User>)), Qt::QueuedConnection);
    // Working with messages signal-slot
    connect(mTelegramService, SIGNAL(messagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)), this, SLOT(onMessagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendGeoPointAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendContactAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendVideoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendAudioAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesSendDocumentAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSetTypingAnswer(qint64,bool)), this, SIGNAL(messagesSetTypingAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetMessagesAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SLOT(onMessagesSearchAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesReadHistoryAnswer(qint64,qint32,qint32,qint32)), this, SIGNAL(messagesReadHistoryAnswer(qint64,qint32,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)), this, SIGNAL(messagesDeleteHistoryAnswer(qint64,qint32,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesRestoreMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesRestoreMessagesAnswer(qint64,QList<qint32>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesReceivedMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesReceivedMessagesAnswer(qint64,QList<qint32>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessageAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SLOT(onMessagesForwardMessagesAnswer(qint64,QList<Message>,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    // Working with chats signal-slots
    connect(mTelegramService, SIGNAL(addChatUserAnswer(qint32,qint32)), this, SIGNAL(addChatUserAnswer(qint32,qint32)), Qt::QueuedConnection);
    // Working with updates signal-slot
    connect(mTelegramService, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), this, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), Qt::QueuedConnection);
    // Working with files signal-slot
    connect(mTelegramService, SIGNAL(uploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)), this, SLOT(onUploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)), Qt::QueuedConnection);
    // Updates
    connect(mTelegramService, SIGNAL(updatesTooLong()), this, SIGNAL(updatesTooLong()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(updateShort(Update,qint32)), this, SLOT(onUpdateShort(Update,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(updatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)), this, SLOT(onUpdatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(updates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)), this, SLOT(onUpdates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)), Qt::QueuedConnection);
    //Miscellaneous
    connect(mTelegramService, SIGNAL(helpGetInviteTextAnswer(qint64,QString)), this, SIGNAL(helpGetInviteTextAnswer(qint64,QString)), Qt::QueuedConnection);

    // Additional
    connect(mTelegramService, SIGNAL(connected()), this, SIGNAL(hasConnected()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(connected()), this, SLOT(onConnected()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(disconnected()), this, SLOT(onDisconnected()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authLoggedIn()), this, SLOT(onAuthLoggedIn()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authLogOutAnswer(qint64,bool)), this, SLOT(onAuthLoggedOut()), Qt::QueuedConnection);
}

RawApiClient::~RawApiClient() {
    mServiceThread->quit();
    mServiceThread->wait();
}
//Miscellaneous
void RawApiClient::helpGetInviteText(const QString &langCode) {
    QMetaObject::invokeMethod(mTelegramService, "helpGetInviteText", Qt::QueuedConnection , Q_ARG(QString, langCode));
}

// Registration / authorization - input operations
void RawApiClient::authCheckPhone() {
    QMetaObject::invokeMethod(mTelegramService, "authCheckPhone", Qt::QueuedConnection);
}
void RawApiClient::authCheckPhone(const QString &phone) {
    QMetaObject::invokeMethod(mTelegramService, "authCheckPhone", Qt::QueuedConnection, Q_ARG(QString, phone));
}

void RawApiClient::authSendCode() {
    QMetaObject::invokeMethod(mTelegramService, "authSendCode", Qt::QueuedConnection);
}

void RawApiClient::authSignIn(const QString &code) {
    QMetaObject::invokeMethod(mTelegramService, "authSignIn", Qt::QueuedConnection, Q_ARG(QString, code));
}

void RawApiClient::authSignUp(const QString &code, const QString &firstName, const QString &lastName) {
    QMetaObject::invokeMethod(mTelegramService, "authSignUp", Qt::QueuedConnection,
                              Q_ARG(QString, code), Q_ARG(QString, firstName), Q_ARG(QString, lastName));
}

void RawApiClient::authSendCall() {
    QMetaObject::invokeMethod(mTelegramService, "authSendCall", Qt::QueuedConnection);
}

void RawApiClient::authLogOut() {
    QMetaObject::invokeMethod(mTelegramService, "authLogOut", Qt::QueuedConnection);
}

void RawApiClient::authResetAuthorizations() {
    QMetaObject::invokeMethod(mTelegramService, "authResetAuthorizations", Qt::QueuedConnection);
}

// Working with notifications and settings - input operations
void RawApiClient::accountRegisterDevice(const QString &token, const QString &appVersion, bool appSandbox) {
    QMetaObject::invokeMethod(mTelegramService, "accountRegisterDevice", Qt::QueuedConnection,
                              Q_ARG(QString, token), Q_ARG(QString, appVersion), Q_ARG(bool, appSandbox));
}

void RawApiClient::accountUnregisterDevice(const QString &token) {
    QMetaObject::invokeMethod(mTelegramService, "accountUnregisterDevice", Qt::QueuedConnection,
                              Q_ARG(QString, token));
}

// Working with contacts
void RawApiClient::contactsGetContacts() {
    QMetaObject::invokeMethod(mTelegramService, "contactsGetContacts", Qt::QueuedConnection);
}

void RawApiClient::contactsImportContacts(const QVariantList &contacts, bool replace) {
    QList<InputContact> contactsList = toQList<InputContact>(contacts);
    QMetaObject::invokeMethod(mTelegramService, "contactsImportContacts", Qt::QueuedConnection,
                              Q_ARG(QList<InputContact>, contactsList), Q_ARG(bool, replace));
}

// Working with messages
void RawApiClient::messagesSendMessage(TLInputPeer *peer, const QString &message) {
    QMetaObject::invokeMethod(mTelegramService, "messagesSendMessage", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer), Q_ARG(QString, message));
}

void RawApiClient::messagesSendPhoto(TLInputPeer *peer, const QString &filePath) {
    QMetaObject::invokeMethod(mTelegramService, "messagesSendPhoto", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer), Q_ARG(QString, filePath));
}

void RawApiClient::sendVideo(qint32 peerId, const QString &filePath, qint32 duration, qint32 width, qint32 height, const QString &thumbnailFilePath) {
    QMetaObject::invokeMethod(mTelegramService, "sendVideo", Qt::QueuedConnection,
                              Q_ARG(qint32, peerId), Q_ARG(QString, filePath), Q_ARG(qint32, duration),
                              Q_ARG(qint32, width), Q_ARG(qint32, height), Q_ARG(QString, thumbnailFilePath));
}

void RawApiClient::messagesGetDialogs(qint32 offset, qint32 maxId, qint32 limit) {
    QMetaObject::invokeMethod(mTelegramService, "messagesGetDialogs", Qt::QueuedConnection,
                              Q_ARG(qint32, offset), Q_ARG(qint32, maxId), Q_ARG(qint32, limit));
}

void RawApiClient::messagesGetHistory(TLInputPeer *peer, qint32 offset, qint32 maxId, qint32 limit) {
    QMetaObject::invokeMethod(mTelegramService, "messagesGetHistory", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer), Q_ARG(qint32, offset), Q_ARG(qint32, maxId), Q_ARG(qint32, limit));
}

void RawApiClient::messagesReadHistory(TLInputPeer *peer, qint32 maxId, qint32 offset) {
    QMetaObject::invokeMethod(mTelegramService, "messagesReadHistory", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer), Q_ARG(qint32, maxId), Q_ARG(qint32, offset));
}

void RawApiClient::messagesDeleteMessages(const QList<qint32> &msgIds, bool isSecret) {
    QMetaObject::invokeMethod(mTelegramService, "messagesDeleteMessages", Qt::QueuedConnection,
                              Q_ARG(QList<qint32>, msgIds), Q_ARG(bool, isSecret));
}

void RawApiClient::deleteChatHistory(TLInputPeer *peer, qint32 offset, bool deleteChat) {
    QMetaObject::invokeMethod(mTelegramService, "deleteChatHistory", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer), Q_ARG(qint32, offset), Q_ARG(bool, deleteChat));
}
void RawApiClient::messagesSetTyping(TLInputPeer *peer) {
    QMetaObject::invokeMethod(mTelegramService, "messagesSetTyping", Qt::QueuedConnection,
                              Q_ARG(InputPeer, *peer));
}

void RawApiClient::messagesSetEncryptedTyping(qint32 chatId, bool typing) {
    QMetaObject::invokeMethod(mTelegramService, "messagesSetEncryptedTyping", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(bool, typing) );
}

void RawApiClient::messagesCreateChat(const QVariantList &users, const QString &title) {

    QList<InputUser> groupUsers = toQList<InputUser>(users);
    QMetaObject::invokeMethod(mTelegramService, "messagesCreateChat", Qt::QueuedConnection,
                              Q_ARG(QList<InputUser>, groupUsers), Q_ARG(QString, title));

}

void RawApiClient::messagesGetFullChat(qint32 chatId) {
    QMetaObject::invokeMethod(mTelegramService, "messagesGetFullChat", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId));
}

void RawApiClient::addChatUser(qint32 chatId, qint32 userId, qint32 fwdLimit) {
    QMetaObject::invokeMethod(mTelegramService, "addChatUser", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(qint32, userId), Q_ARG(qint32, fwdLimit));
}

void RawApiClient::deleteChatUser(qint32 chatId, qint32 userId) {
    QMetaObject::invokeMethod(mTelegramService, "deleteChatUser", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(qint32, userId));
}

// Working with users - received signals
void RawApiClient::onUsersGetUsersAnswer(qint64 id, const QList<User> &users) {
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT usersGetUsersAnswer(id, usersList);
}

void RawApiClient::onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName) {
    TLUser *tlUser = new TLUser(this);
    tlUser->setData(&user);
    TLContactsLink *tlLink = new TLContactsLink(this);
    tlLink->setData(&link);
    TLPhoto *tlProfilePhoto = new TLPhoto(this);
    tlProfilePhoto->setData(&profilePhoto);
    TLPeerNotifySettings *tlNotifySettings = new TLPeerNotifySettings(this);
    tlNotifySettings->setData(&notifySettings);
    Q_EMIT usersGetFullUserAnswer(id, tlUser, tlLink, tlProfilePhoto, tlNotifySettings, blocked, realFirstName, realLastName);
}

void RawApiClient::onPhotosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, const QList<Photo> &photos, const QList<User> &users) {
    QVariantList photosList = toVariantList<Photo, TLPhoto>(photos);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT photosGetUserPhotosAnswer(id, sliceCount, photosList, usersList);
}

// Working with contacts - received signals
void RawApiClient::onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses) {
    QVariantList statusesList = toVariantList<ContactStatus, TLContactStatus>(statuses);
    Q_EMIT contactsGetStatusesAnswer(id, statusesList);
}

void RawApiClient::onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users) {
    QVariantList contactsList = toVariantList<Contact, TLContact>(contacts);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsGetContactsAnswer(id, modified, contactsList, usersList);
}

void RawApiClient::onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &importedContacts, const QList<qint64> &retryContacts, const QList<User> &users) {
    QVariantList importedContactsList = toVariantList<ImportedContact, TLImportedContact>(importedContacts);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsImportContactsAnswer(id, importedContactsList, retryContacts, usersList);
}

void RawApiClient::onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user) {
    TLContactsMyLink *tlMyLink = new TLContactsMyLink(this);
    tlMyLink->setData(&myLink);
    TLContactsForeignLink *tlForeignLink = new TLContactsForeignLink(this);
    tlForeignLink->setData(&foreignLink);
    TLUser *tlUser = new TLUser(this);
    tlUser->setData(&user);
    Q_EMIT contactsDeleteContactAnswer(id, tlMyLink, tlForeignLink, tlUser);
}

// Working with blacklist - received signals
void RawApiClient::onContactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QList<ContactBlocked> blocked, QList<User> users) {
    QVariantList blockedList = toVariantList<ContactBlocked, TLContactBlocked>(blocked);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT contactsGetBlockedAnswer(id, sliceCount, blockedList, usersList);
}

// Working with messages - received signals
void RawApiClient::onMessagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links) {
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    Q_EMIT messagesSendMessageAnswer(id, msgId, date, pts, seq, linksList);
}

void RawApiClient::onMessagesSendMediaAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendMediaAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendPhotoAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendGeoPointAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendContactAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendVideoAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendAudioAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSendDocumentAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesGetMessagesAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetMessagesAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesGetDialogsAnswer(qint64 id, qint32 sliceCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    // emit single message of received answer to get dialogs operation
    QVariantList dialogsList = toVariantList<Dialog, TLDialog>(dialogs);
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetDialogsAnswer(id, sliceCount, dialogsList, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesGetHistoryAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesGetHistoryAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesSearchAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesSearchAnswer(id, sliceCount, messagesList, chatsList, usersList);
}

void RawApiClient::onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    TLMessage *msg = new TLMessage(this);
    msg->setData(&message);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesForwardMessageAnswer(id, msg, chatsList, usersList, linksList, pts, seq);
}

void RawApiClient::onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq) {
    QVariantList messagesList = toVariantList<Message, TLMessage>(messages);
    QVariantList linksList = toVariantList<ContactsLink, TLContactsLink>(links);
    QVariantList chatsList = toVariantList<Chat, TLChat>(chats);
    QVariantList usersList = toVariantList<User, TLUser>(users);
    Q_EMIT messagesForwardMessagesAnswer(id, messagesList, chatsList, usersList, linksList, pts, seq);
}

// Working with files - received signals
void RawApiClient::onUploadGetFileAnswer(qint64 id, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total) {
    TLStorageFileType *tlType = new TLStorageFileType(this);
    tlType->setData(&type);
    Q_EMIT uploadGetFileAnswer(id, tlType, mtime, bytes, partId, downloaded, total);
}


void RawApiClient::onUpdateShort(const Update &update, qint32 date) {
    TLUpdate *tlUpdate = new TLUpdate(this);
    tlUpdate->setData(&update);
    Q_EMIT updateShort(tlUpdate, date);
}

void RawApiClient::onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq) {
}

void RawApiClient::onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq) {
}

void RawApiClient::onConnected() {
    Q_EMIT connectedChanged(true);
}

void RawApiClient::onDisconnected() {
    Q_EMIT connectedChanged(false);
}

void RawApiClient::onAuthLoggedIn() {
    Q_EMIT authenticatedChanged(true);
}

void RawApiClient::onAuthLoggedOut() {
    Q_EMIT authenticatedChanged(false);
}


TelegramService *RawApiClient::service() {
    return mTelegramService;
}

QString RawApiClient::phoneNumber() {
    QString result;
    QMetaObject::invokeMethod(mTelegramService, "phoneNumber", Qt::BlockingQueuedConnection, Q_RETURN_ARG(QString, result));
    return result;
}

void RawApiClient::setPhoneNumber(const QString &phoneNumber) {
    QMetaObject::invokeMethod(mTelegramService, "setPhoneNumber", Qt::QueuedConnection, Q_ARG(QString, phoneNumber));
    Q_EMIT phoneNumberChanged(phoneNumber);
}

QString RawApiClient::configPath() {
    QString result;
    QMetaObject::invokeMethod(mTelegramService, "configPath", Qt::BlockingQueuedConnection, Q_RETURN_ARG(QString, result));
    return result;
}

void RawApiClient::setConfigPath(const QString &configPath) {
    QMetaObject::invokeMethod(mTelegramService, "setConfigPath", Qt::QueuedConnection, Q_ARG(QString, configPath));
    Q_EMIT configPathChanged(configPath);
}

QString RawApiClient::publicKeyFile() {
    QString result;
    QMetaObject::invokeMethod(mTelegramService, "publicKeyFile", Qt::BlockingQueuedConnection, Q_RETURN_ARG(QString, result));
    return result;
}

void RawApiClient::setPublicKeyFile(const QString &publicKeyFile) {
    QMetaObject::invokeMethod(mTelegramService, "setPublicKeyFile", Qt::QueuedConnection, Q_ARG(QString, publicKeyFile));
    Q_EMIT publicKeyFileChanged(publicKeyFile);
}

bool RawApiClient::connected() {
    bool result;
    QMetaObject::invokeMethod(mTelegramService, "isConnected", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, result));
    return result;
}

bool RawApiClient::authenticated() {
    bool result;
    QMetaObject::invokeMethod(mTelegramService, "isAuthenticated", Qt::BlockingQueuedConnection, Q_RETURN_ARG(bool, result));
    return result;
}

template <typename TOrig, typename TDest>
QVariantList RawApiClient::toVariantList(const QList<TOrig> &list) {
    QVariantList newList;
    int id = qMetaTypeId<TDest>();
    Q_ASSERT(id);
    Q_FOREACH(const TOrig &item, list) {
        TDest *typeObject = (TDest *)QMetaType::create(id);
        TLObject *tlObject = (TLObject *)typeObject;
        tlObject->setData(&item);
        newList.append(qVariantFromValue(typeObject));
    }
    return newList;
}

template <typename T>
QList<T> RawApiClient::toQList(const QVariantList &variantList) {
    QList<T> tList;
    for (int i = 0; i < variantList.size(); i++) {
        T *element = dynamic_cast<T *>(variantList.at(i).value<QObject *>());
        tList.append(*element);
    }
    return tList;
}

