/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef RAWAPICLIENT_H
#define RAWAPICLIENT_H

#include <QVariantList>
#include <QObject>
#include <QString>
#include <QSortFilterProxyModel>
#include <telegram.h>
#include <types/types.h>
#include "config.h"
#include "types/tlaudio.h"
#include "types/tlchat.h"
#include "types/tlchatparticipant.h"
#include "types/tlchatparticipants.h"
#include "types/tlchatphoto.h"
#include "types/tlcontact.h"
#include "types/tlcontactblocked.h"
#include "types/tlcontactsforeignlink.h"
#include "types/tlcontactslink.h"
#include "types/tlcontactsmylink.h"
#include "types/tlcontactstatus.h"
#include "types/tldcoption.h"
#include "types/tldialog.h"
#include "types/tldocument.h"
#include "types/tlencryptedchat.h"
#include "types/tlencryptedfile.h"
#include "types/tlencryptedmessage.h"
#include "types/tlfilelocation.h"
#include "types/tlgeochatmessage.h"
#include "types/tlgeopoint.h"
#include "types/tlimportedcontact.h"
#include "types/tlinputcontact.h"
#include "types/tlinputfilelocation.h"
#include "types/tlinputgeopoint.h"
#include "types/tlinputpeer.h"
#include "types/tlinputuser.h"
#include "types/tlmessage.h"
#include "types/tlmessageaction.h"
#include "types/tlmessagemedia.h"
#include "types/tlmessagesfilter.h"
#include "types/tlnotifypeer.h"
#include "types/tlpeer.h"
#include "types/tlpeernotifysettings.h"
#include "types/tlphoto.h"
#include "types/tlphotosize.h"
#include "types/tlstoragefiletype.h"
#include "types/tlupdate.h"
#include "types/tlupdatesstate.h"
#include "types/tluserprofilephoto.h"
#include "types/tluserstatus.h"
#include "types/tluser.h"
#include "types/tlvideo.h"
#include "servicethread.h"
#include "telegramservice.h"


class RawApiClient : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QString phoneNumber READ phoneNumber WRITE setPhoneNumber NOTIFY phoneNumberChanged)
    Q_PROPERTY(QString configPath READ configPath WRITE setConfigPath NOTIFY configPathChanged)
    Q_PROPERTY(QString publicKeyFile READ publicKeyFile WRITE setPublicKeyFile NOTIFY publicKeyFileChanged)
    Q_PROPERTY(bool connected READ connected NOTIFY connectedChanged)
    Q_PROPERTY(bool authenticated READ authenticated NOTIFY authenticatedChanged)
public:
    explicit RawApiClient(QObject *parent = 0);
    ~RawApiClient();

    TelegramService *service();

    // Registration / authorization
    Q_INVOKABLE void authCheckPhone();
    Q_INVOKABLE void authCheckPhone(const QString &phone);
    Q_INVOKABLE void authSendCode();
    Q_INVOKABLE void authSignIn(const QString &code);
    Q_INVOKABLE void authSignUp(const QString &code, const QString &firstName, const QString &lastName);
    Q_INVOKABLE void authSendCall();

    Q_INVOKABLE void authLogOut();
    Q_INVOKABLE void authResetAuthorizations();
    // Working with notifications and settings
    Q_INVOKABLE void accountRegisterDevice(const QString &token, const QString &appVersion = "", bool appSandbox = false);
    Q_INVOKABLE void accountUnregisterDevice(const QString &token);
    // Working with contacts
    Q_INVOKABLE void contactsGetContacts();
    Q_INVOKABLE void contactsImportContacts(const QVariantList &contacts, bool replace = false);

    // Working with messages
    Q_INVOKABLE void messagesSendMessage(TLInputPeer *peer, const QString &message);
    Q_INVOKABLE void messagesSendPhoto(TLInputPeer *peer, const QString &filePath);
    Q_INVOKABLE void sendVideo(qint32 peerId, const QString &filePath, qint32 duration = 0, qint32 width = 0, qint32 height = 0, const QString &thumbnailFilePath = "");
    Q_INVOKABLE void messagesGetDialogs(qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    Q_INVOKABLE void messagesGetHistory(TLInputPeer *peer, qint32 offset = 0, qint32 maxId = 0, qint32 limit = 0);
    Q_INVOKABLE void messagesReadHistory(TLInputPeer *peer, qint32 maxId = 0, qint32 offset = 0);
    Q_INVOKABLE void messagesDeleteMessages(const QList<qint32> &msgIds, bool isSecret = false);
    Q_INVOKABLE void deleteChatHistory(TLInputPeer *peer, qint32 offset = 0, bool deleteChat = false);
    Q_INVOKABLE void messagesSetTyping(TLInputPeer *peer);
    Q_INVOKABLE void messagesSetEncryptedTyping(qint32 chatId, bool typing);


    // Working with chats
    Q_INVOKABLE void messagesCreateChat(const QVariantList &users, const QString &title);
    Q_INVOKABLE void messagesGetFullChat(qint32 chatId);
    Q_INVOKABLE void addChatUser(qint32 chatId, qint32 userId, qint32 fwdLimit = 0);
    Q_INVOKABLE void deleteChatUser(qint32 chatId, qint32 userId);

    Q_INVOKABLE QString phoneNumber();
    Q_INVOKABLE QString configPath();
    Q_INVOKABLE QString publicKeyFile();
    Q_INVOKABLE bool connected();
    Q_INVOKABLE bool authenticated();
    Q_INVOKABLE void setPhoneNumber(const QString &phoneNumber);
    Q_INVOKABLE void setConfigPath(const QString &configPath);
    Q_INVOKABLE void setPublicKeyFile(const QString &publicKeyFile);

    //Miscellaneous
    Q_INVOKABLE void helpGetInviteText(const QString &langCode);

Q_SIGNALS:
    void hasConnected();
    // Errors
    void fatalError();
    void error(qint64 id, qint32 errorCode, QString errorText);
    void authSignInError(qint64 id, qint32 errorCode, QString errorText);
    void authSignUpError(qint64 id, qint32 errorCode, QString errorText);
    // Registration / authorization
    void authNeeded();
    void authLoggedIn();
    void authCheckPhoneAnswer(qint64 id, bool phoneRegistered, bool phoneInvited);
    void authCheckPhoneSent(qint64 msgId, QString phoneNumber);
    void authSendCodeAnswer(qint64 id, bool phoneRegistered, qint32 sendCallTimeout);
    void authSendCallAnswer(qint64 id, bool ok);
    void authLogOutAnswer(qint64 id, bool ok);
    void authSendInvitesAnswer(qint64 id, bool ok);
    void authResetAuthorizationsAnswer(qint64 id, bool ok);
    // Working with notifications and settings
    void accountRegisterDeviceAnswer(qint64 id, bool ok);
    void accountUnregisterDeviceAnswer(qint64 id, bool ok);
    // Working with users
    void usersGetUsersAnswer(qint64 id, QVariantList users);
    void usersGetFullUserAnswer(qint64 id, TLUser *user, TLContactsLink *link, TLPhoto *profilePhoto, TLPeerNotifySettings *notifySettings, bool blocked, QString realFirstName, QString realLastName);
    void photosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, QVariantList photos, QVariantList users);
    // Working with contacts
    void contactsGetStatusesAnswer(qint64 id, QVariantList statuses);
    void contactsGetContactsAnswer(qint64 id, bool modified, QVariantList contacts, QVariantList users);
    void contactsImportContactsAnswer(qint64 id, QVariantList importedContacts, QList<qint64> retryContacts, QVariantList users);
    void contactsDeleteContactAnswer(qint64 id, TLContactsMyLink *myLink, TLContactsForeignLink *foreignLink, TLUser *user);
    void contactsDeleteContactsAnswer(qint64 id, bool ok);
    // Working with blacklist
    void contactsBlockAnswer(qint64 id, bool ok);
    void contactsUnblockAnswer(qint64 id, bool ok);
    void contactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QVariantList blocked, QVariantList users);
    // Working with messages
    void messagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, QVariantList links);
    void messagesSendMediaAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendPhotoAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendGeoPointAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendContactAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendVideoAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendAudioAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSendDocumentAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesSetTypingAnswer(qint64 id, bool ok);
    void messagesGetMessagesAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesGetDialogsAnswer(qint64 id, qint32 sliceCount, QVariantList dialogs, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesGetHistoryAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesSearchAnswer(qint64 id, qint32 sliceCount, QVariantList messages, QVariantList chats, QVariantList users);
    void messagesReadHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void messagesDeleteHistoryAnswer(qint64 id, qint32 pts, qint32 seq, qint32 offset);
    void messagesDeleteMessagesAnswer(qint64 id, QList<qint32> deletedMsgIds);
    void messagesRestoreMessagesAnswer(qint64 id, QList<qint32> restoredMsgIds);
    void messagesReceivedMessagesAnswer(qint64 id, QList<qint32> confirmedMsgIds);
    void messagesForwardMessageAnswer(qint64 id, TLMessage *message, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);
    void messagesForwardMessagesAnswer(qint64 id, QVariantList messages, QVariantList chats, QVariantList users, QVariantList links, qint32 pts, qint32 seq);    
    // Working with chats
    void messagesGetChatsAnswer(qint64 id, QList<Chat> chats, QList<User> users);
    void messagesGetFullChatAnswer(qint64 id, ChatFull chatFull, QList<Chat> chats, QList<User> users);
    void messagesEditChatTitleAnswer(qint64 id, Message message, QList<Chat> chats, QList<User> users, QList<ContactsLink> links, qint32 pts, qint32 seq);// statedMessage or statedMessageLink
    void messagesEditChatPhotoAnswer(qint64 id, Message message, QList<Chat> chats, QList<User> users, QList<ContactsLink> links, qint32 pts, qint32 seq); // statedMessage or statedMessageLink
    void addChatUserAnswer(qint32 chatId, qint32 userId);
    void messagesDeleteChatUserAnswer(qint64 id, Message message, QList<Chat> chats, QList<User> users, QList<ContactsLink> links, qint32 pts, qint32 seq); // statedMessage or statedMessageLink
    void messagesCreateChatAnswer(qint64 id, Message message, QList<Chat> chats, QList<User> users, QList<ContactsLink> links, qint32 pts, qint32 seq); // statedMessage or statedMessageLink
    // Working with updates
    void updatesGetStateAnswer(qint64 id, qint32 pts, qint32 qts, qint32 date, qint32 seq, qint32 unreadCount);
    void updatesGetDifferenceAnswer(qint64 id, qint32 date, qint32 seq);
    void updatesGetDifferenceAnswer(qint64 id, QVariantList messages, QVariantList newEncryptedMessages, QVariantList otherUpdates, QVariantList chats, QVariantList users, TLUpdatesState *state, bool isIntermediateState);
    // Working with files
    void uploadGetFileAnswer(qint64 id, TLStorageFileType *type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);
    // Updates
    void updatesTooLong();
    void updateShortMessage(qint32 id, qint32 fromId, QString message, qint32 pts, qint32 date, qint32 seq);
    void updateShortChatMessage(qint32 id, qint32 fromId, qint32 chatId, QString message, qint32 pts, qint32 date, qint32 seq);
    void updateShort(TLUpdate *update, qint32 date);
    void updatesCombined(QVariantList updates, QVariantList users, QVariantList chats, qint32 date, qint32 seqStart, qint32 seq);
    void updates(QVariantList udts, QVariantList users, QVariantList chats, qint32 date, qint32 seq);

    void phoneNumberChanged(QString);
    void configPathChanged(QString);
    void publicKeyFileChanged(QString);
    void connectedChanged(bool);
    void authenticatedChanged(bool);

    //Miscellaneous
    void helpGetInviteTextAnswer(qint64 id, QString message);

protected Q_SLOTS:
    // Working with users
    void onUsersGetUsersAnswer(qint64 id, const QList<User> &users);
    void onUsersGetFullUserAnswer(qint64 id, const User &user, const ContactsLink &link, const Photo &profilePhoto, const PeerNotifySettings &notifySettings, bool blocked, const QString &realFirstName, const QString &realLastName);
    void onPhotosGetUserPhotosAnswer(qint64 id, qint32 sliceCount, const QList<Photo> &photos, const QList<User> &users);
    // Working with contacts
    void onContactsGetStatusesAnswer(qint64 id, const QList<ContactStatus> &statuses);
    void onContactsGetContactsAnswer(qint64 id, bool modified, const QList<Contact> &contacts, const QList<User> &users);
    void onContactsImportContactsAnswer(qint64 id, const QList<ImportedContact> &importedContacts, const QList<qint64> &retryContacts, const QList<User> &users);
    void onContactsDeleteContactAnswer(qint64 id, const ContactsMyLink &myLink, const ContactsForeignLink &foreignLink, const User &user);
    // Working with blacklist
    void onContactsGetBlockedAnswer(qint64 id, qint32 sliceCount, QList<ContactBlocked> blocked, QList<User> users);
    // Working with messages
    void onMessagesSendMessageAnswer(qint64 id, qint32 msgId, qint32 date, qint32 pts, qint32 seq, const QList<ContactsLink> &links);
    void onMessagesSendMediaAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendPhotoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendGeoPointAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendContactAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendVideoAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendAudioAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendDocumentAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesGetMessagesAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetDialogsAnswer(qint64 id, qint32 sliceCount, const QList<Dialog> &dialogs, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesGetHistoryAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesSearchAnswer(qint64 id, qint32 sliceCount, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users);
    void onMessagesForwardMessageAnswer(qint64 id, const Message &message, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesForwardMessagesAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    void onMessagesSendBroadcastAnswer(qint64 id, const QList<Message> &messages, const QList<Chat> &chats, const QList<User> &users, const QList<ContactsLink> &links, qint32 pts, qint32 seq);
    // Working with files
    void onUploadGetFileAnswer(qint64 id, const StorageFileType &type, qint32 mtime, QByteArray bytes, qint32 partId, qint32 downloaded, qint32 total);
    // Updates
    void onUpdateShort(const Update &update, qint32 date);
    void onUpdatesCombined(const QList<Update> &updates, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seqStart, qint32 seq);
    void onUpdates(const QList<Update> &udts, const QList<User> &users, const QList<Chat> &chats, qint32 date, qint32 seq);

    void onConnected();
    void onDisconnected();
    void onAuthLoggedIn();
    void onAuthLoggedOut();

protected:
    ServiceThread *mServiceThread;
    TelegramService *mTelegramService;

private:
    /**
     * Converts signal received QList<T> data into QVariantList to be used from qml
     */
    template <typename TOrig, typename TDest>
    static QVariantList toVariantList(const QList<TOrig> &list);
    /**
     * Converts QVariantList to QList
     */
    template <typename T>
    QList<T> toQList(const QVariantList &variantList);
};

#endif // RAWAPICLIENT_H
