/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Roberto Mier
 *      Tiago Herrmann
 */
#include "rawapiservice.h"
#include "config.h"

RawApiService::RawApiService() :
    QObject(),
    mTelegramLib(0) {
}

RawApiService::~RawApiService(){
    if (mTelegramLib) {
        mTelegramLib->deleteLater();
        mTelegramLib = 0;
    }
}

Config &RawApiService::config() {
    return mConfig;
}

void RawApiService::initTelegramLibrary() {

    Q_ASSERT(mConfig.phoneNumber().length() > 0);

    if (!mTelegramLib) {
        if (mConfig.configPath().isEmpty()) {
            mTelegramLib = new Telegram(mConfig.phoneNumber());
        } else if (mConfig.publicKeyFile().isEmpty()) {
            mTelegramLib = new Telegram(mConfig.phoneNumber(), mConfig.configPath());
        } else {
            mTelegramLib = new Telegram(mConfig.phoneNumber(), mConfig.configPath(), mConfig.publicKeyFile());
        }

        //propagate signals to have them available in telegram client

        // Errors
        connect(mTelegramLib, SIGNAL(fatalError()), this, SIGNAL(fatalError()));
        connect(mTelegramLib, SIGNAL(error(qint64,qint32,QString)), this, SIGNAL(error(qint64,qint32,QString)));
        connect(mTelegramLib, SIGNAL(authSignInError(qint64,qint32,QString)), this, SIGNAL(authSignInError(qint64,qint32,QString)));
        connect(mTelegramLib, SIGNAL(authSignUpError(qint64,qint32,QString)), this, SIGNAL(authSignUpError(qint64,qint32,QString)));
        // Registration / authorization signal-slot
        connect(mTelegramLib, SIGNAL(authNeeded()), this, SIGNAL(authNeeded()));
        connect(mTelegramLib, SIGNAL(authLoggedIn()), this, SIGNAL(authLoggedIn()));
        connect(mTelegramLib, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), this, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)));
        connect(mTelegramLib, SIGNAL(authCheckPhoneSent(qint64, QString)), this, SIGNAL(authCheckPhoneSent(qint64, QString)));
        connect(mTelegramLib, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), this, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)));
        connect(mTelegramLib, SIGNAL(authSendCallAnswer(qint64,bool)), this, SIGNAL(authSendCallAnswer(qint64,bool)));
        connect(mTelegramLib, SIGNAL(authLogOutAnswer(qint64,bool)), this, SIGNAL(authLogOutAnswer(qint64,bool)));
        connect(mTelegramLib, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)), this, SIGNAL(authResetAuthorizationsAnswer(qint64,bool)));
        // Working with notifications and settings
        connect(mTelegramLib, SIGNAL(accountRegisterDeviceAnswer(qint64,bool)), this, SIGNAL(accountRegisterDeviceAnswer(qint64,bool)));
        connect(mTelegramLib, SIGNAL(accountUnregisterDeviceAnswer(qint64,bool)), this, SIGNAL(accountUnregisterDeviceAnswer(qint64,bool)));
         // Working with messages signal-slot
        connect(mTelegramLib, SIGNAL(messagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)), this, SIGNAL(messagesSendMessageAnswer(qint64,qint32,qint32,qint32,qint32,QList<ContactsLink>)));
        connect(mTelegramLib, SIGNAL(messagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SIGNAL(messagesSendMediaAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(messagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), this, SIGNAL(messagesSendPhotoAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(messagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)), this, SIGNAL(messagesGetDialogsAnswer(qint64,qint32, QList<Dialog>,QList<Message>,QList<Chat>,QList<User>)));
        connect(mTelegramLib, SIGNAL(messagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)), this, SIGNAL(messagesGetHistoryAnswer(qint64,qint32,QList<Message>,QList<Chat>,QList<User>)));
        connect(mTelegramLib, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)), this, SIGNAL(messagesDeleteMessagesAnswer(qint64,QList<qint32>)));
        connect(mTelegramLib, SIGNAL(messagesReadEncryptedHistoryAnswer(qint64,bool)), this, SIGNAL(messagesReadEncryptedHistoryAnswer(qint64,bool)));

        // Working with chats
        connect(mTelegramLib, SIGNAL(messagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), SIGNAL(messagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)));

        // Working with contacts
        connect(mTelegramLib, SIGNAL(contactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)), this, SIGNAL(contactsImportContactsAnswer(qint64,QList<ImportedContact>,QList<qint64>,QList<User>)));

        // Working with updates signal-slot
        connect(mTelegramLib, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)), this, SIGNAL(updatesGetStateAnswer(qint64,qint32,qint32,qint32,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)), this, SIGNAL(updatesGetDifferenceAnswer(qint64,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(updatesGetDifferenceAnswer(qint64,QList<Message>,QList<SecretChatMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)), this, SIGNAL(updatesGetDifferenceAnswer(qint64,QList<Message>,QList<SecretChatMessage>,QList<Update>,QList<Chat>,QList<User>,UpdatesState,bool)));
        // Working with files signal-slot
        connect(mTelegramLib, SIGNAL(uploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)), this, SIGNAL(uploadGetFileAnswer(qint64,StorageFileType,qint32,QByteArray,qint32,qint32,qint32)));
        // Updates
        connect(mTelegramLib, SIGNAL(updatesTooLong()), this, SIGNAL(updatesTooLong()));
        connect(mTelegramLib, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortMessage(qint32,qint32,QString,qint32,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)), this, SIGNAL(updateShortChatMessage(qint32,qint32,qint32,QString,qint32,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(updateShort(Update,qint32)), this, SIGNAL(updateShort(Update,qint32)));
        connect(mTelegramLib, SIGNAL(updatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)), this, SIGNAL(updatesCombined(QList<Update>,QList<User>,QList<Chat>,qint32,qint32,qint32)));
        connect(mTelegramLib, SIGNAL(updates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)), this, SIGNAL(updates(QList<Update>,QList<User>,QList<Chat>,qint32,qint32)));

        //Miscellaneous
        connect(mTelegramLib, SIGNAL(helpGetInviteTextAnswer(qint64,QString)), this, SIGNAL(helpGetInviteTextAnswer(qint64,QString)));

        // Additional
        connect(mTelegramLib, SIGNAL(connected()), this, SIGNAL(connected()));
        connect(mTelegramLib, SIGNAL(disconnected()), this, SIGNAL(disconnected()));
        connect(mTelegramLib, SIGNAL(woken()), this, SIGNAL(woken()));

    } else {
        mTelegramLib->setPhoneNumber(mConfig.phoneNumber());
    }

    // Phone number after set in lib is filtered (stripped non digits or + chars), so assign its filtered value to qmlplugin config
    mConfig.setPhoneNumber(Settings::getInstance()->phoneNumber());
}


QString RawApiService::phoneNumber() const {
    return mConfig.phoneNumber();
}

QString RawApiService::configPath() const {
    return mConfig.configPath();
}

QString RawApiService::publicKeyFile() const {
    return mConfig.publicKeyFile();
}

bool RawApiService::isConnected() {
    if (mTelegramLib) {
        return mTelegramLib->isConnected();
    }
    return false;
}

bool RawApiService::isAuthenticated() {
    if (mTelegramLib) {
        return mTelegramLib->isLoggedIn();
    }
    return false;
}

void RawApiService::setPhoneNumber(const QString &phoneNumber) {
    mConfig.setPhoneNumber(phoneNumber);
}

void RawApiService::setConfigPath(const QString &configPath) {
    mConfig.setConfigPath(configPath);
}

void RawApiService::setPublicKeyFile(const QString &publicKeyFile) {
    mConfig.setPublicKeyFile(publicKeyFile);
}

qint64 RawApiService::helpGetInviteText(const QString &langCode) {
    return mTelegramLib->helpGetInviteText(langCode);
}

qint64 RawApiService::authSendInvites(const QStringList &phoneNumbers, const QString &message){
     return mTelegramLib->authSendInvites(phoneNumbers,message);
}

qint64 RawApiService::authResetAuthorizations() {
    return mTelegramLib->authResetAuthorizations();
}

qint64 RawApiService::authCheckPhone() {
    return mTelegramLib->authCheckPhone();
}
qint64 RawApiService::authCheckPhone(const QString &phone){
    return mTelegramLib->authCheckPhone(phone);
}
qint64 RawApiService::authSendCode() {
    return mTelegramLib->authSendCode();
}

qint64 RawApiService::authSignIn(const QString &code) {
    return mTelegramLib->authSignIn(code);
}

qint64 RawApiService::authSignUp(const QString &code, const QString &firstName, const QString &lastName) {
    return mTelegramLib->authSignUp(code, firstName, lastName);
}

qint64 RawApiService::authSendCall() {
    return mTelegramLib->authSendCall();
}

qint64 RawApiService::authLogOut() {
    return mTelegramLib->authLogOut();
}

qint64 RawApiService::accountRegisterDevice(const QString &token, const QString &appVersion, bool appSandbox) {
    return mTelegramLib->accountRegisterDevice(token, appVersion, appSandbox);
}

qint64 RawApiService::accountUnregisterDevice(const QString &token) {
    return mTelegramLib->accountUnregisterDevice(token);
}

qint64 RawApiService::usersGetUsers(const QList<InputUser> &users) {
    return mTelegramLib->usersGetUsers(users);
}

qint64 RawApiService::contactsImportContacts(const QList<InputContact> &contacts, bool replace) {
    return mTelegramLib->contactsImportContacts(contacts, replace);
}

qint64 RawApiService::contactsGetContacts() {
    return mTelegramLib->contactsGetContacts();
}

qint64 RawApiService::messagesSendMessage(const InputPeer &peer, qint64 randomId, const QString &message) {
    return mTelegramLib->messagesSendMessage(peer, randomId, message);
}

qint64 RawApiService::messagesSendPhoto(const InputPeer &peer, qint64 randomId, const QString &filePath) {
    return mTelegramLib->messagesSendPhoto(peer, randomId, filePath);
}

qint64 RawApiService::messagesSendVideo(const InputPeer &peer, qint64 randomId, const QString &filePath, qint32 duration, qint32 width, qint32 height, const QString &thumbnailFilePath) {
    return mTelegramLib->messagesSendVideo(peer, randomId, filePath, duration, width, height, thumbnailFilePath);
}

qint64 RawApiService::messagesGetDialogs(qint32 offset, qint32 maxId, qint32 limit) {
    return mTelegramLib->messagesGetDialogs(offset, maxId, limit);
}

qint64 RawApiService::messagesGetHistory(const InputPeer &peer, qint32 offset, qint32 maxId, qint32 limit) {
    return mTelegramLib->messagesGetHistory(peer, offset, maxId, limit);
}

qint64 RawApiService::messagesReadHistory(const InputPeer &peer, qint32 maxId, qint32 offset, bool readContents) {
    return mTelegramLib->messagesReadHistory(peer, maxId, offset, readContents);
}

qint64 RawApiService::messagesDeleteMessages(const QList<qint32> &msgIds) {
    return mTelegramLib->messagesDeleteMessages(msgIds);
}

qint64 RawApiService::messagesDeleteHistory(const InputPeer &peer, qint32 offset) {
    return mTelegramLib->messagesDeleteHistory(peer, offset);
}
qint64 RawApiService::messagesSetTyping(const InputPeer &peer, const SendMessageAction &action) {
    return mTelegramLib->messagesSetTyping(peer, action) ;
}

qint64 RawApiService::messagesSetEncryptedTyping(qint32 chatId, bool typing) {
    return mTelegramLib->messagesSetEncryptedTyping(chatId,typing);
}

qint64 RawApiService::messagesCreateChat(const QList<InputUser> &users, const QString &chatTopic)  {
    return mTelegramLib->messagesCreateChat(users, chatTopic);
}

qint64 RawApiService::messagesGetFullChat(qint32 chatId) {
    return mTelegramLib->messagesGetFullChat(chatId);
}

qint64 RawApiService::messagesAddChatUser(qint32 chatId, const InputUser &user, qint32 fwdLimit) {
    return mTelegramLib->messagesAddChatUser(chatId, user, fwdLimit);
}

qint64 RawApiService::messagesDeleteChatUser(qint32 chatId, const InputUser &user) {
    return mTelegramLib->messagesDeleteChatUser(chatId, user);
}

// Working with secret chats
qint64 RawApiService::messagesReceivedQueue(qint32 maxQts) {
    return mTelegramLib->messagesReceivedQueue(maxQts);
}

qint64 RawApiService::messagesReadEncryptedHistory(qint32 chatId, qint32 maxDate) {
    return mTelegramLib->messagesReadEncryptedHistory(chatId, maxDate);
}

// Working with updates
qint64 RawApiService::updatesGetState() {
    return mTelegramLib->updatesGetState();
}

qint64 RawApiService::updatesGetDifference(qint32 pts, qint32 date, qint32 qts) {
    return mTelegramLib->updatesGetDifference(pts, date, qts);
}

// Working with files
qint64 RawApiService::uploadGetFile(const InputFileLocation &file, qint32 fileSize, qint32 dc, const QByteArray &key, const QByteArray &iv) {
    return mTelegramLib->uploadGetFile(file, fileSize, dc, key, iv);
}
