/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Roberto Mier
 *      Tiago Herrmann
 */
#include "telegramclient.h"
#include <QMimeDatabase>

TelegramClient::TelegramClient(QObject *parent) :
    RawApiClient(parent), mChatToOpen(0)
{
    // register types for having queued connection in signal-slots
    qRegisterMetaType<QList<ContactItem> >("QList<ContactItem>");
    qRegisterMetaType<QList<DialogItem> >("QList<DialogItem>");
    qRegisterMetaType<QList<MessageItem> >("QList<MessageItem>");
    qRegisterMetaType<ContactItem>("ContactItem");
    qRegisterMetaType<InputPeer>("InputPeer");

    // connect signal-slots linking service thread with qml UI
    connect(mTelegramService, SIGNAL(ourIdChanged(qint32)), SIGNAL(ourIdChanged(qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(databasePathChanged(QString)), SIGNAL(databasePathChanged(QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(openChat()), SLOT(openChat()));

    connect(mTelegramService, SIGNAL(error(qint64,qint32,QString)), SIGNAL(error(qint64,qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSignInError(qint64,qint32,QString)), SIGNAL(authSignInError(qint64,qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSignUpError(qint64,qint32,QString)), SIGNAL(authSignUpError(qint64,qint32,QString)), Qt::QueuedConnection);

    connect(mTelegramService, SIGNAL(authNeeded()), SIGNAL(authNeeded()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authLoggedIn()), SIGNAL(authLoggedIn()), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), SIGNAL(authCheckPhoneAnswer(qint64,bool,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), SIGNAL(authSendCodeAnswer(qint64,bool,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(authSendCallAnswer(qint64,bool)), SIGNAL(authSendCallAnswer(qint64,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(woken()), SIGNAL(woken()), Qt::QueuedConnection);

    connect(mTelegramService, SIGNAL(messagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), SLOT(onMessagesCreateChatAnswer(qint64,Message,QList<Chat>,QList<User>,QList<ContactsLink>,qint32,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(getUserAnswer(ContactItem)), SLOT(onGetUserAnswer(ContactItem)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(addContactsAnswer(QList<qint32>)), SIGNAL(addContactsAnswer(QList<qint32>)), Qt::QueuedConnection);

    // connect signal-signal comming from data updates
    connect(mTelegramService, SIGNAL(contactsAdded(QList<ContactItem>)), SIGNAL(contactsAdded(QList<ContactItem>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactThumbnailUpdated(qint32,QString)), SIGNAL(contactThumbnailUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactBigPhotoUpdated(qint32,QString)), SIGNAL(contactBigPhotoUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(chatBigPhotoUpdated(qint32,QString)), SIGNAL(chatBigPhotoUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(dialogThumbnailUpdated(qint32,QString)), SIGNAL(dialogThumbnailUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(dialogTitleUpdated(qint32,QString)), SIGNAL(dialogTitleUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(dialogMuted(qint32,bool)), SIGNAL(dialogMuted(qint32,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(dialogsMuted(NotifyPeer::NotifyPeerType,bool)), SIGNAL(dialogsMuted(NotifyPeer::NotifyPeerType,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messageAdded(MessageItem)), SIGNAL(messageAdded(MessageItem)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesAdded(QList<MessageItem>,qint32)), SIGNAL(messagesAdded(QList<MessageItem>,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(outgoingMessageAdded(qint32,MessageItem)), SIGNAL(outgoingMessageAdded(qint32,MessageItem)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messageUpdated(qint32,MessageItem)), SIGNAL(messageUpdated(qint32,MessageItem)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(chatMarkedAsRead(qint32)), SIGNAL(chatMarkedAsRead(qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(chatHistoryDeleted(qint32)), SIGNAL(chatHistoryDeleted(qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(chatDeleted(qint32)), SIGNAL(chatDeleted(qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(chatUpdated(qint32)), SIGNAL(chatUpdated(qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesMarkedAsRead(QList<qint32>)), SIGNAL(messagesMarkedAsRead(QList<qint32>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesDeleted(QList<qint32>)), SIGNAL(messagesDeleted(QList<qint32>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagePhotoUpdated(qint32,QString,bool)), SIGNAL(messagePhotoUpdated(qint32,QString,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messageVideoUpdated(qint32,QString)), SIGNAL(messageVideoUpdated(qint32,QString)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(downloadingUpdated(qint32,bool)), SIGNAL(downloadingUpdated(qint32,bool)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(downloadedPercentageUpdated(qint32,qint8)), SIGNAL(downloadedPercentageUpdated(qint32,qint8)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(dialogsAdded(QList<DialogItem>,qint32)), SIGNAL(dialogsAdded(QList<DialogItem>,qint32)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(userTyping(qint32,QString)), SIGNAL(userTyping(qint32,QString)), Qt::QueuedConnection);

    // views signal-signal
    connect(mTelegramService, SIGNAL(dialogsNextPage(QList<DialogItem>)), SIGNAL(dialogsNextPage(QList<DialogItem>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(messagesNextPage(QList<MessageItem>)), SIGNAL(messagesNextPage(QList<MessageItem>)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(contactsNextPage(QList<ContactItem>)), SIGNAL(contactsNextPage(QList<ContactItem>)), Qt::QueuedConnection);

    // secret chats
    connect(mTelegramService, SIGNAL(secretChatRequested(qint32,qint32,qint32,qint64)), SIGNAL(secretChatRequested(qint32,qint32,qint32,qint64)), Qt::QueuedConnection);
    connect(mTelegramService, SIGNAL(secretChatCreated(qint32)), SIGNAL(secretChatCreated(qint32)));
    connect(mTelegramService, SIGNAL(secretChatDiscarded(qint32)), SIGNAL(secretChatDiscarded(qint32)));
    connect(mTelegramService, SIGNAL(secretChatUpdated(qint32)), SIGNAL(secretChatUpdated(qint32)));

    connect(mTelegramService, SIGNAL(unreadCountChanged(qint32)), SIGNAL(unreadCountChanged(qint32)));

    // connect database signals
    DbManager &dbManager = mTelegramService->data().dbManager();
    connect(&dbManager, SIGNAL(initialized()), SIGNAL(databaseInitialized()));

    // connect the rest of the thread signals with this signals
    mServiceThread->start();
}

TelegramClient::~TelegramClient() {
}

QString TelegramClient::fileMimeType(const QString &fileName) {
    QMimeDatabase db;
    QMimeType type = db.mimeTypeForFile(fileName);
    return type.name();
}

qint32 TelegramClient::ourId() {
    qint32 result;
    QMetaObject::invokeMethod(mTelegramService, "ourId", Qt::BlockingQueuedConnection, Q_RETURN_ARG(qint32, result));
    return result;
}

QString TelegramClient::databasePath() const {
    QString result;
    QMetaObject::invokeMethod(mTelegramService, "databasePath", Qt::BlockingQueuedConnection, Q_RETURN_ARG(QString, result));
    return result;
}
void TelegramClient::setDatabasePath(const QString &path) {
    QMetaObject::invokeMethod(mTelegramService, "setDatabasePath", Qt::QueuedConnection, Q_ARG(QString, path));
}

QString TelegramClient::downloadsPath() const {
    QString result;
    QMetaObject::invokeMethod(mTelegramService, "downloadsPath", Qt::BlockingQueuedConnection, Q_RETURN_ARG(QString, result));
    return result;
}

void TelegramClient::setDownloadsPath(const QString &path) {
    QMetaObject::invokeMethod(mTelegramService, "setDownloadsPath", Qt::QueuedConnection, Q_ARG(QString, path));
}

qint32 TelegramClient::getChatToOpen() const {
    return mChatToOpen;
}

void TelegramClient::setChatToOpen(qint32 chatToOpen) {
    mChatToOpen = chatToOpen;
}

void TelegramClient::init() {
    QMetaObject::invokeMethod(mTelegramService, "init", Qt::QueuedConnection);
}

void TelegramClient::initTelegramLibrary() {
    QMetaObject::invokeMethod(mTelegramService, "initTelegramLibrary", Qt::QueuedConnection);
}

void TelegramClient::shutdown() {
    QMetaObject::invokeMethod(mTelegramService, "shutdown", Qt::QueuedConnection);
}

void TelegramClient::sleep() {
    QMetaObject::invokeMethod(mTelegramService, "sleep", Qt::QueuedConnection);
}

void TelegramClient::wake() {
    QMetaObject::invokeMethod(mTelegramService, "wake", Qt::QueuedConnection);
}


void TelegramClient::initializeDialogsView() {
    QMetaObject::invokeMethod(mTelegramService, "initializeDialogsView", Qt::QueuedConnection);
}

void TelegramClient::initializeMessagesView(qint32 dialogId, qint32 peerType) {
    QMetaObject::invokeMethod(mTelegramService, "initializeMessagesView",
                              Qt::QueuedConnection,
                              Q_ARG(qint32, dialogId),
                              Q_ARG(qint32, peerType));
}

void TelegramClient::initializeContactsView() {
    QMetaObject::invokeMethod(mTelegramService, "initializeContactsView", Qt::QueuedConnection);
}

void TelegramClient::fetchDialogsNextPage() {
    QMetaObject::invokeMethod(mTelegramService, "fetchDialogsNextPage", Qt::QueuedConnection);
}

void TelegramClient::fetchMessagesNextPage() {
    QMetaObject::invokeMethod(mTelegramService, "fetchMessagesNextPage", Qt::QueuedConnection);
}

void TelegramClient::fetchContactsNextPage() {
    QMetaObject::invokeMethod(mTelegramService, "fetchContactsNextPage", Qt::QueuedConnection);
}

void TelegramClient::onMessagesCreateChatAnswer(qint64, const Message &, const QList<Chat> &chats, const QList<User> &, const QList<ContactsLink> &,qint32, qint32) {
    Q_FOREACH (const Chat &chat, chats) {
        TLChat tlChat;
        tlChat.setData(&chat);
        QVariant chatVariant = QVariant::fromValue(&tlChat);
        Q_EMIT messagesCreateChatAnswer(chatVariant);
    }
}

void TelegramClient::addContact(const QString &firstName, const QString &lastName, const QString &phone) {
    QMetaObject::invokeMethod(mTelegramService, "addContact", Qt::QueuedConnection,
                              Q_ARG(QString, firstName), Q_ARG(QString, lastName), Q_ARG(QString, phone));
}

void TelegramClient::getUser(qint32 userId) {
    QMetaObject::invokeMethod(mTelegramService, "getUser", Qt::QueuedConnection, Q_ARG(qint32, userId));
}

void TelegramClient::createSecretChat(qint32 userId) {
    QMetaObject::invokeMethod(mTelegramService, "createSecretChat", Qt::QueuedConnection, Q_ARG(qint32, userId));
}

void TelegramClient::acceptSecretChat(qint32 chatId) {
    QMetaObject::invokeMethod(mTelegramService, "acceptSecretChat", Qt::QueuedConnection, Q_ARG(qint32, chatId));
}

void TelegramClient::discardSecretChat(qint32 chatId) {
    QMetaObject::invokeMethod(mTelegramService, "discardSecretChat", Qt::QueuedConnection, Q_ARG(qint32, chatId));
}

void TelegramClient::sendSecretChatMessage(qint32 chatId, const QString &message, qint32 ttl) {
    QMetaObject::invokeMethod(mTelegramService, "sendSecretChatMessage", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(QString, message), Q_ARG(qint32, ttl));
}

void TelegramClient::sendSecretChatPhoto(qint32 chatId, const QString &filePath, qint32 ttl) {
    QMetaObject::invokeMethod(mTelegramService, "sendSecretChatPhoto", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(QString, filePath), Q_ARG(qint32, ttl));
}

void TelegramClient::sendSecretChatVideo(qint32 chatId, const QString &filePath, qint32 ttl) {
    QMetaObject::invokeMethod(mTelegramService, "sendSecretChatVideo", Qt::QueuedConnection,
                              Q_ARG(qint32, chatId), Q_ARG(QString, filePath), Q_ARG(qint32, ttl));
}

void TelegramClient::readSecretChatHistory(qint32 chatId) {
    QMetaObject::invokeMethod(mTelegramService, "readSecretChatHistory", Qt::QueuedConnection, Q_ARG(qint32, chatId));
}

void TelegramClient::deleteSecretChatHistory(qint32 dialogId, bool deleteChat) {
    QMetaObject::invokeMethod(mTelegramService, "deleteSecretChatHistory", Qt::QueuedConnection,
                              Q_ARG(qint32, dialogId), Q_ARG(bool, deleteChat));
}

void TelegramClient::setSecretChatTTL(qint32 dialogId, qint32 ttl) {
    QMetaObject::invokeMethod(mTelegramService, "setSecretChatTTL", Qt::QueuedConnection,
                              Q_ARG(qint32, dialogId), Q_ARG(qint32, ttl));
}

void TelegramClient::downloadPhoto(qint32 messageId) {
    QMetaObject::invokeMethod(mTelegramService, "downloadPhoto", Qt::QueuedConnection, Q_ARG(qint32, messageId));
}

void TelegramClient::downloadVideo(qint32 messageId) {
    QMetaObject::invokeMethod(mTelegramService, "downloadVideo", Qt::QueuedConnection, Q_ARG(qint32, messageId));
}

void TelegramClient::downloadUserBigPhoto(qint32 userId) {
    QMetaObject::invokeMethod(mTelegramService, "downloadUserBigPhoto", Qt::QueuedConnection, Q_ARG(qint32, userId));
}

void TelegramClient::downloadChatBigPhoto(qint32 chatId) {
    QMetaObject::invokeMethod(mTelegramService, "downloadChatBigPhoto", Qt::QueuedConnection, Q_ARG(qint32, chatId));
}

void TelegramClient::cancelDownload(qint32 id) {
    QMetaObject::invokeMethod(mTelegramService, "cancelDownload", Qt::QueuedConnection, Q_ARG(qint32, id));
}

void TelegramClient::mute(qint32 peerId) {
    QMetaObject::invokeMethod(mTelegramService, "mute", Qt::QueuedConnection, Q_ARG(qint32, peerId));
}

void TelegramClient::unmute(qint32 peerId) {
    QMetaObject::invokeMethod(mTelegramService, "unmute", Qt::QueuedConnection, Q_ARG(qint32, peerId));
}

void TelegramClient::authSendInvite(const QString &phoneNumber, const QString &message) {
    QMetaObject::invokeMethod(mTelegramService, "authSendInvite", Qt::QueuedConnection, Q_ARG(QString, phoneNumber), Q_ARG(QString, message));
}

void TelegramClient::onGetUserAnswer(ContactItem info) {
    QVariant variant = QVariant::fromValue(&info);
    Q_EMIT getUserAnswer(variant);
}

void TelegramClient::openChat() {
    if (mChatToOpen != 0) {
        qint32 chatId = mChatToOpen;
        mChatToOpen = 0;

        Q_EMIT openChat(chatId);
    }
}

void TelegramClient::onPushNotifications(const QStringList &) {
    qCDebug(TG_PLUGIN_LOGIC) << "push notifications list received";
    // do nothing with notifications list. If app is alive they will be also received in app,
    // if app is not alive, app will be launched by user interaction with notifications and
    // update state will be asked directly to the servers
}

void TelegramClient::onPushError(const QString &error) {
    qCWarning(TG_PLUGIN_LOGIC) << "push notification error:" << error;
}

qint32 TelegramClient::getDialogIdFromMessageId(qint32 msgId) {
    qint32 result;
    QMetaObject::invokeMethod(mTelegramService, "getDialogIdFromMessageId", Qt::BlockingQueuedConnection,
                              Q_RETURN_ARG(qint32, result),
                              Q_ARG(qint32, msgId));
    return result;
}

MessageItem TelegramClient::topMessage(qint32 dialogId) {
    MessageItem result;
    QMetaObject::invokeMethod(mTelegramService, "topMessage", Qt::BlockingQueuedConnection,
                              Q_RETURN_ARG(MessageItem, result),
                              Q_ARG(qint32, dialogId));
    return result;
}

qint32 TelegramClient::getUnreadCount(qint32 dialogId) {
    qint32 result;
    QMetaObject::invokeMethod(mTelegramService, "getUnreadCount", Qt::BlockingQueuedConnection,
                              Q_RETURN_ARG(qint32, result),
                              Q_ARG(qint32, dialogId));
    return result;
}

