/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <QtQml>
#include <QtQml/QQmlContext>
#include "telegramplugin.h"
#include "telegramclient.h"
#include "types/tldialog.h"
#include "types/tluser.h"
#include "types/tlchat.h"
#include "types/tlchatparticipant.h"
#include "types/tlchatparticipants.h"
#include "types/tlcontact.h"
#include "types/tlcontactblocked.h"
#include "types/tlcontactsforeignlink.h"
#include "types/tlcontactslink.h"
#include "types/tlcontactstatus.h"
#include "types/tldcoption.h"
#include "types/tlencryptedchat.h"
#include "types/tlencryptedfile.h"
#include "types/tlencryptedmessage.h"
#include "types/tlgeochatmessage.h"
#include "types/tlimportedcontact.h"
#include "types/tlnotifypeer.h"
#include "types/tlstoragefiletype.h"
#include "types/tlupdate.h"
#include "types/tlinputcontact.h"
#include "types/tlinputfilelocation.h"
#include "types/tlinputgeopoint.h"
#include "types/tlinputpeer.h"
#include "types/tlinputuser.h"
#include "types/tlmessagesfilter.h"
#include "types/tlmessage.h"
#include "models/model.h"
#include "models/dialogmodel.h"
#include "models/messagesmodel.h"
#include "models/contactsmodel.h"
#include "models/contactsproxy.h"
#include "models/groupmodel.h"
#include "models/groupmembersmodel.h"
#include "models/dialogsmodel.h"
#include "models/dialogsproxy.h"
#include "models/sortproxymodel.h"

void TelegramPlugin::registerTypes(const char *uri)
{
    //@uri Ubuntu.Plugins.Telegram
    Q_ASSERT(uri == QLatin1String("Ubuntu.Plugins.Telegram"));

    // Telegram qml main type
    qmlRegisterType<TelegramClient>(uri, 0, 1, "TelegramClient");

    // Output qml data types
    qmlRegisterType<TLAudio>(uri, 0, 1, "TLAudio");
    qmlRegisterType<TLChat>(uri, 0, 1, "TLChat");
    qmlRegisterType<TLChatParticipant>(uri, 0, 1, "TLChatParticipant");
    qmlRegisterType<TLChatParticipants>(uri, 0, 1, "TLChatParticipants");
    qmlRegisterType<TLChatPhoto>(uri, 0, 1, "TLChatPhoto");
    qmlRegisterType<TLContact>(uri, 0, 1, "TLContact");
    qmlRegisterType<TLContactBlocked>(uri, 0, 1, "TLContactBlocked");
    qmlRegisterType<TLContactsForeignLink>(uri, 0, 1, "TLContactsForeignLink");
    qmlRegisterType<TLContactsLink>(uri, 0, 1, "TLContactsLink");
    qmlRegisterType<TLContactsMyLink>(uri, 0, 1, "TLContactsMyLink");
    qmlRegisterType<TLContactStatus>(uri, 0, 1, "TLContactStatus");
    qmlRegisterType<TLDcOption>(uri, 0, 1, "TLDcOption");
    qmlRegisterType<TLDialog>(uri, 0, 1, "TLDialog");
    qmlRegisterType<TLDocument>(uri, 0, 1, "TLDocument");
    qmlRegisterType<TLEncryptedChat>(uri, 0, 1, "TLEncryptedChat");
    qmlRegisterType<TLEncryptedFile>(uri, 0, 1, "TLEncryptedFile");
    qmlRegisterType<TLEncryptedMessage>(uri, 0, 1, "TLEncryptedMessage");
    qmlRegisterType<TLFileLocation>(uri, 0, 1, "TLFileLocation");
    qmlRegisterType<TLGeoChatMessage>(uri, 0, 1, "TLGeoChatMessage");
    qmlRegisterType<TLGeoPoint>(uri, 0, 1, "TLGeoPoint");
    qmlRegisterType<TLImportedContact>(uri, 0, 1, "TLImportedContact");
    qmlRegisterType<TLMessage>(uri, 0, 1, "TLMessage");
    qmlRegisterType<TLMessageAction>(uri, 0, 1, "TLMessageAction");
    qmlRegisterType<TLMessageMedia>(uri, 0, 1, "TLMessageMedia");
    qmlRegisterType<TLNotifyPeer>(uri, 0, 1, "TLNotifyPeer");
    qmlRegisterType<TLPeer>(uri, 0, 1, "TLPeer");
    qmlRegisterType<TLPeerNotifySettings>(uri, 0, 1, "TLPeerNotifySettings");
    qmlRegisterType<TLPhoto>(uri, 0, 1, "TLPhoto");
    qmlRegisterType<TLPhotoSize>(uri, 0, 1, "TLPhotoSize");
    qmlRegisterType<TLStorageFileType>(uri, 0, 1, "TLStorageFileType");
    qmlRegisterType<TLUpdate>(uri, 0, 1, "TLUpdate");
    qmlRegisterType<TLUser>(uri, 0, 1, "TLUser");
    qmlRegisterType<TLUserProfilePhoto>(uri, 0, 1, "TLUserProfilePhoto");
    qmlRegisterType<TLUserStatus>(uri, 0, 1, "TLUserStatus");
    qmlRegisterType<TLVideo>(uri, 0, 1, "TLVideo");

    // Input qml data types
    qmlRegisterType<TLInputContact>(uri, 0, 1, "TLInputContact");
    qmlRegisterType<TLInputFileLocation>(uri, 0, 1, "TLInputFileLocation");
    qmlRegisterType<TLInputGeoPoint>(uri, 0, 1, "TLInputGeoPoint");
    qmlRegisterType<TLInputPeer>(uri, 0, 1, "TLInputPeer");
    qmlRegisterType<TLInputUser>(uri, 0, 1, "TLInputUser");
    qmlRegisterType<TLMessagesFilter>(uri, 0, 1, "TLMessagesFilter");

    // Models
    qmlRegisterType<QSortFilterProxyModel>(uri, 0, 1, "QSortFilterProxyModel");
    qmlRegisterType<DialogsModel>(uri, 0, 1, "DialogsModel");
    qmlRegisterType<DialogsProxy>(uri, 0, 1, "DialogsProxy");
    qmlRegisterType<DialogModel>(uri, 0, 1, "DialogModel");
    qmlRegisterType<MessagesModel>(uri, 0, 1, "MessagesModel");
    qmlRegisterType<ContactsModel>(uri, 0, 1, "ContactsModel");
    qmlRegisterType<ContactsProxy>(uri, 0, 1, "ContactsProxy");
    qmlRegisterType<GroupModel>(uri, 0, 1, "GroupModel");
    qmlRegisterType<GroupMembersModel>(uri, 0, 1, "GroupMembersModel");
    qmlRegisterType<DialogItem>(uri, 0, 1, "DialogItem");
    qmlRegisterType<MessageItem>(uri, 0, 1, "MessageItem");
    qmlRegisterType<ContactItem>(uri, 0, 1, "ContactItem");
    qmlRegisterType<SortProxyModel>(uri, 0, 1, "SortProxyModel");
}

void TelegramPlugin::initializeEngine(QQmlEngine *engine, const char *uri)
{
    QQmlExtensionPlugin::initializeEngine(engine, uri);
}
