/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authors:
 *      Roberto Mier
 *      Tiago Herrmann
 */
#ifndef TOOLS_H
#define TOOLS_H

#include "types/types.h"
#include "models/contactitem.h"

#include <thumbnailer.h>

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(TG_PLUGIN_LOGIC)

class Tools
{
public:

    static InputUser toInputUser(const ContactItem &user);
    static InputPeer toInputPeer(qint32 peerId, const ContactItem &user);
    static InputUser::InputUserType toInputUserType(User::UserType userType);
    static InputPeer::InputPeerType toInputPeerType(User::UserType userType);
    static QString generateThumbnail(const QString &videoPath);
    static QString getExtensionFromMimeType(const QString &mimeType);
    static qint64 generateRandomId();
    static qint32 requestIdToMsgId(qint64 requestId);
    static MessageMedia::MessageMediaType toMessageMediaType(DecryptedMessageMedia::DecryptedMessageMediaType decryptedMessageMediaType);
    static qint32 getBelongingDialogId(const Message &message);
    static QString pngToJpeg(const QString &pngFile, qint32 quality = -1);
    static void resizePhotoIfNeeded(const QString &photoFilePath);
};

#endif // TOOLS_H
