/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLAUDIO_H
#define TLAUDIO_H

#include "tlobject.h"
#include <types/audio.h>

class TLAudio : public TLObject, public Audio {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLAudioType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(QString mimeType READ mimeType)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 dcId READ dcId)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(qint32 duration READ duration)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint64 id READ id)
    Q_PROPERTY(qint32 size READ size)

public:
    enum TLAudioType {
       TypeAudio = Audio::typeAudio,
       TypeAudioEmpty = Audio::typeAudioEmpty
    };

    explicit TLAudio(QObject *parent = 0) : 
        TLObject(parent), 
        Audio(Audio::typeAudioEmpty) {
    }
    TLAudio(const TLAudio &other) :
        TLObject(other.parent()),
        Audio(other.classType()) {
        setData(&other);
    }
    ~TLAudio() {}

    void setData(const void *data) {
        Audio *obj = (Audio *)data;
        setClassType(obj->classType());
        setMimeType(obj->mimeType());
        setAccessHash(obj->accessHash());
        setDcId(obj->dcId());
        setUserId(obj->userId());
        setDuration(obj->duration());
        setDate(obj->date());
        setId(obj->id());
        setSize(obj->size());
    }
};
Q_DECLARE_METATYPE(TLAudio)

#endif // TLAUDIO_H
