/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLCHATPARTICIPANTS_H
#define TLCHATPARTICIPANTS_H

#include "tlobject.h"
#include "tlchatparticipant.h"
#include <types/chatparticipants.h>
#include <QQmlListProperty>

class TLChatParticipants : public TLObject, public ChatParticipants {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLChatParticipantsType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 adminId READ adminId)
    Q_PROPERTY(QQmlListProperty<TLChatParticipant> participants READ participants)
    Q_PROPERTY(qint32 version READ version)
    Q_PROPERTY(qint32 chatId READ chatId)

public:
    enum TLChatParticipantsType {
        TypeChatParticipants = ChatParticipants::typeChatParticipants,
        TypeChatParticipantsForbidden = ChatParticipants::typeChatParticipantsForbidden
    };

    explicit TLChatParticipants(QObject *parent = 0) : 
        TLObject(parent), 
        ChatParticipants(ChatParticipants::typeChatParticipants) {
    }
    TLChatParticipants(const TLChatParticipants &other) :
        TLObject(other.parent()),
        ChatParticipants(other.classType()) {
        setData(&other);
    }
    ~TLChatParticipants() {}

    void setData(const void *data) {
        ChatParticipants *obj = (ChatParticipants *)data;
        setClassType(obj->classType());
        setAdminId(obj->adminId());
        setVersion(obj->version());
        setChatId(obj->chatId());
        Q_FOREACH (ChatParticipant s, obj->participants()) {
            TLChatParticipant *tlChatParticipant = new TLChatParticipant(this);
            tlChatParticipant->setData(&s);
            m_participants.append(tlChatParticipant);
        }
    }

    QQmlListProperty<TLChatParticipant> participants() {
        return QQmlListProperty<TLChatParticipant>(this, m_participants);
    }

private:
    QList<TLChatParticipant *> m_participants;
};
Q_DECLARE_METATYPE(TLChatParticipants)

#endif // TLCHATPARTICIPANTS_H
