/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLCHATPHOTO_H
#define TLCHATPHOTO_H

#include "tlobject.h"
#include "tlfilelocation.h"
#include <types/chatphoto.h>

class TLChatPhoto : public TLObject, public ChatPhoto {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLChatPhotoType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLFileLocation *photoSmall READ photoSmall)
    Q_PROPERTY(TLFileLocation *photoBig READ photoBig)

public:
    enum TLChatPhotoType {
        TypeChatPhoto = ChatPhoto::typeChatPhoto,
        TypeChatPhotoEmpty = ChatPhoto::typeChatPhotoEmpty
    };

    explicit TLChatPhoto(QObject *parent = 0) : 
        TLObject(parent), 
        ChatPhoto(ChatPhoto::typeChatPhotoEmpty) {
    }
    TLChatPhoto(const TLChatPhoto &other) :
        TLObject(other.parent()),
        ChatPhoto(other.classType()) {
        setData(&other);
    }
    ~TLChatPhoto() {}

    void setData(const void *data) {
        ChatPhoto *obj = (ChatPhoto *)data;
        setClassType(obj->classType());
        setPhotoSmall(obj->photoSmall());
        setPhotoBig(obj->photoBig());
    }

    void setPhotoSmall(const FileLocation &photoSmall) { m_photoSmall.setData(&photoSmall); }
    void setPhotoBig(const FileLocation &photoBig) { m_photoBig.setData(&photoBig); }
    TLFileLocation *photoSmall() { return &m_photoSmall; }
    TLFileLocation *photoBig() { return &m_photoBig; }

private:
    TLFileLocation m_photoSmall;
    TLFileLocation m_photoBig;
};
Q_DECLARE_METATYPE(TLChatPhoto)

#endif // TLCHATPHOTO_H
