/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLCONTACTSLINK_H
#define TLCONTACTSLINK_H

#include "tlobject.h"
#include "tluser.h"
#include "tlcontactsmylink.h"
#include "tlcontactsforeignlink.h"
#include <types/contactslink.h>

class TLContactsLink : public TLObject, public ContactsLink {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLContactsLinkType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLContactsMyLink *myLink READ myLink)
    Q_PROPERTY(TLContactsForeignLink *foreignLink READ foreignLink)
    Q_PROPERTY(TLUser *user READ user)

public:
    enum TLContactsLinkType {
        TypeContactsLink = ContactsLink::typeContactsLink
    };

    explicit TLContactsLink(QObject *parent = 0) : 
        TLObject(parent) {
    }
    TLContactsLink(const TLContactsLink &other) :
        TLObject(other.parent()) {
        setData(&other);
    }
    ~TLContactsLink() {}

    void setData(const void *data) {
        ContactsLink *obj = (ContactsLink *)data;
        setClassType(obj->classType());
        setMyLink(obj->myLink());
        setForeignLink(obj->foreignLink());
        setUser(obj->user());
    }

    void setMyLink(const ContactsMyLink &myLink) { m_myLink.setData(&myLink); }
    void setForeignLink(const ContactsForeignLink &foreignLink) { m_foreignLink.setData(&foreignLink); }
    void setUser(const User &user) { m_user.setData(&user); }
    TLContactsMyLink *myLink() { return &m_myLink; }
    TLContactsForeignLink *foreignLink() { return &m_foreignLink; }
    TLUser *user() { return &m_user; }

private:
    TLContactsMyLink m_myLink;
    TLContactsForeignLink m_foreignLink;
    TLUser m_user;
};
Q_DECLARE_METATYPE(TLContactsLink)

#endif // TLCONTACTSLINK_H
