/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QMLDIALOG_H
#define QMLDIALOG_H

#include <types/dialog.h>
#include <types/peer.h>
#include <types/peernotifysettings.h>
#include "tlobject.h"
#include "tlpeer.h"
#include "tlpeernotifysettings.h"

class TLDialog : public TLObject, public Dialog {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLDialogType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLPeer *peer READ peer)
    Q_PROPERTY(qint32 topMessage READ topMessage)
    Q_PROPERTY(TLPeerNotifySettings *notifySettings READ notifySettings)
    Q_PROPERTY(qint32 unreadCount READ unreadCount)
public:
    enum TLDialogType {
        TypeDialog = Dialog::typeDialog
    };

    explicit TLDialog(QObject *parent = 0) : TLObject(parent) {}
    TLDialog(const TLDialog &other) :
        TLObject(other.parent()) {
        setData(&other);
    }
    ~TLDialog() {}

    void setData(const void *data) {
        Dialog *d = (Dialog *)data;
        setTopMessage(d->topMessage());
        setUnreadCount(d->unreadCount());
        setPeer(d->peer());
        setNotifySettings(d->notifySettings());
        setClassType(d->classType());
    }

    void setPeer(const Peer &peer) { m_peer.setData(&peer); }
    void setNotifySettings(const PeerNotifySettings &notifySettings) { m_notifySettings.setData(&notifySettings); }
    TLPeer *peer() { return &m_peer; }
    TLPeerNotifySettings *notifySettings() { return &m_notifySettings; }

private:
    TLPeer m_peer;
    TLPeerNotifySettings m_notifySettings;
};
Q_DECLARE_METATYPE(TLDialog)

#endif // QMLDIALOG_H
