/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLDOCUMENT_H
#define TLDOCUMENT_H

#include "tlobject.h"
#include "tlphotosize.h"
#include <types/document.h>

class TLDocument : public TLObject, public Document {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLDocumentType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(QString mimeType READ mimeType)
    Q_PROPERTY(TLPhotoSize *thumb READ thumb)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 dcId READ dcId)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(QString fileName READ fileName)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint64 id READ id)
    Q_PROPERTY(qint32 size READ size)

public:
    enum TLDocumentType {
        TypeDocument = Document::typeDocument,
        TypeDocumentEmpty = Document::typeDocumentEmpty
    };

    explicit TLDocument(QObject *parent = 0) : 
        TLObject(parent), 
        Document(Document::typeDocumentEmpty) {
    }
    TLDocument(const TLDocument &other) :
        TLObject(other.parent()),
        Document(other.classType()) {
        setData(&other);
    }
    ~TLDocument() {}

    void setData(const void *data) {
        Document *obj = (Document *)data;
        setClassType(obj->classType());
        setMimeType(obj->mimeType());
        setThumb(obj->thumb());
        setAccessHash(obj->accessHash());
        setDcId(obj->dcId());
        setUserId(obj->userId());
        setFileName(obj->fileName());
        setDate(obj->date());
        setId(obj->id());
        setSize(obj->size());
    }

    void setThumb(const PhotoSize &thumb) { m_thumb.setData(&thumb); }
    TLPhotoSize *thumb() { return &m_thumb; }

private:
    TLPhotoSize m_thumb;
};
Q_DECLARE_METATYPE(TLDocument)

#endif // TLDOCUMENT_H
