/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLENCRYPTEDCHAT_H
#define TLENCRYPTEDCHAT_H

#include "tlobject.h"
#include <types/encryptedchat.h>

class TLEncryptedChat : public TLObject, public EncryptedChat {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLEncryptedChatType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 participantId READ participantId)
    Q_PROPERTY(QByteArray gAOrB READ gAOrB)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 adminId READ adminId)
    Q_PROPERTY(qint64 keyFingerprint READ keyFingerprint)
    Q_PROPERTY(QByteArray gA READ gA)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint32 id READ id)

public:
    enum TLEncryptedChatType {
        TypeEncryptedChatWaiting = EncryptedChat::typeEncryptedChatWaiting,
        TypeEncryptedChatRequested = EncryptedChat::typeEncryptedChatRequested,
        TypeEncryptedChatEmpty = EncryptedChat::typeEncryptedChatEmpty,
        TypeEncryptedChatDiscarded = EncryptedChat::typeEncryptedChatDiscarded,
        TypeEncryptedChat = EncryptedChat::typeEncryptedChat
    };

    explicit TLEncryptedChat(QObject *parent = 0) : 
        TLObject(parent), 
        EncryptedChat(EncryptedChat::typeEncryptedChatEmpty) {
    }
    TLEncryptedChat(const TLEncryptedChat &other) :
        TLObject(other.parent()),
        EncryptedChat(other.classType()) {
        setData(&other);
    }
    ~TLEncryptedChat() {}

    void setData(const void *data) {
        EncryptedChat *obj = (EncryptedChat *)data;
        setClassType(obj->classType());
        setParticipantId(obj->participantId());
        setGAOrB(obj->gAOrB());
        setAccessHash(obj->accessHash());
        setAdminId(obj->adminId());
        setKeyFingerprint(obj->keyFingerprint());
        setGA(obj->gA());
        setDate(obj->date());
        setId(obj->id());
    }
};
Q_DECLARE_METATYPE(TLEncryptedChat)

#endif // TLENCRYPTEDCHAT_H
