/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLENCRYPTEDFILE_H
#define TLENCRYPTEDFILE_H

#include "tlobject.h"
#include <types/encryptedfile.h>

class TLEncryptedFile : public TLObject, public EncryptedFile {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLEncryptedFileType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 keyFingerprint READ keyFingerprint)
    Q_PROPERTY(qint32 dcId READ dcId)
    Q_PROPERTY(qint64 id READ id)
    Q_PROPERTY(qint32 size READ size)

public:
    enum TLEncryptedFileType {
        TypeEncryptedFileEmpty = EncryptedFile::typeEncryptedFileEmpty,
        TypeEncryptedFile = EncryptedFile::typeEncryptedFile
    };

    explicit TLEncryptedFile(QObject *parent = 0) : 
        TLObject(parent), 
        EncryptedFile(EncryptedFile::typeEncryptedFileEmpty) {
    }
    TLEncryptedFile(const TLEncryptedFile &other) :
        TLObject(other.parent()),
        EncryptedFile(other.classType()) {
        setData(&other);
    }
    ~TLEncryptedFile() {}

    void setData(const void *data) {
        EncryptedFile *obj = (EncryptedFile *)data;
        setClassType(obj->classType());
        setAccessHash(obj->accessHash());
        setKeyFingerprint(obj->keyFingerprint());
        setDcId(obj->dcId());
        setId(obj->id());
        setSize(obj->size());
    }
};
Q_DECLARE_METATYPE(TLEncryptedFile)

#endif // TLENCRYPTEDFILE_H
