/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLENCRYPTEDMESSAGE_H
#define TLENCRYPTEDMESSAGE_H

#include "tlobject.h"
#include "tlencryptedfile.h"
#include <types/encryptedmessage.h>

class TLEncryptedMessage : public TLObject, public EncryptedMessage {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLEncryptedMessageType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(QByteArray bytes READ bytes)
    Q_PROPERTY(TLEncryptedFile *file READ file)
    Q_PROPERTY(qint64 randomId READ randomId)
    Q_PROPERTY(qint32 chatId READ chatId)

public:
    enum TLEncryptedMessageType {
        TypeEncryptedMessageService = EncryptedMessage::typeEncryptedMessageService,
        TypeEncryptedMessage = EncryptedMessage::typeEncryptedMessage
    };

    explicit TLEncryptedMessage(QObject *parent = 0) : 
        TLObject(parent), 
        EncryptedMessage(EncryptedMessage::typeEncryptedMessage) {
    }
    TLEncryptedMessage(const TLEncryptedMessage &other) :
        TLObject(other.parent()),
        EncryptedMessage(other.classType()) {
        setData(&other);
    }
    ~TLEncryptedMessage() {}

    void setData(const void *data) {
        EncryptedMessage *obj = (EncryptedMessage *)data;
        setClassType(obj->classType());
        setDate(obj->date());
        setBytes(obj->bytes());
        setFile(obj->file());
        setRandomId(obj->randomId());
        setChatId(obj->chatId());
    }

    void setFile(const EncryptedFile &file) { mFile.setData(&file); }
    TLEncryptedFile *file() { return &mFile; }

private:
    TLEncryptedFile mFile;
};
Q_DECLARE_METATYPE(TLEncryptedMessage)

#endif // TLENCRYPTEDMESSAGE_H
