/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLFILELOCATION_H
#define TLFILELOCATION_H

#include <types/filelocation.h>
#include "tlobject.h"

class TLFileLocation : public TLObject, public FileLocation {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLFileLocationType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint64 secret READ secret)
    Q_PROPERTY(qint32 dcId READ dcId)
    Q_PROPERTY(qint64 volumeId READ volumeId)
    Q_PROPERTY(qint32 localId READ localId)

public:
    enum TLFileLocationType {
        TypeFileLocation = FileLocation::typeFileLocation,
        TypeFileLocationUnavailable = FileLocation::typeFileLocationUnavailable
    };

    explicit TLFileLocation(QObject *parent = 0) : 
        TLObject(parent), 
        FileLocation(FileLocation::typeFileLocationUnavailable) {
    }
    TLFileLocation(const TLFileLocation &other) :
        TLObject(other.parent()),
        FileLocation(other.classType()) {
        setData(&other);
    }
    ~TLFileLocation() {}

    void setData(const void *data) {
        FileLocation *obj = (FileLocation *)data;
        setClassType(obj->classType());
        setSecret(obj->secret());
        setDcId(obj->dcId());
        setVolumeId(obj->volumeId());
        setLocalId(obj->localId());
    }
};
Q_DECLARE_METATYPE(TLFileLocation)

#endif // TLFILELOCATION_H
