/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLGEOCHATMESSAGE_H
#define TLGEOCHATMESSAGE_H

#include "tlobject.h"
#include "tlmessagemedia.h"
#include "tlmessageaction.h"
#include <types/geochatmessage.h>

class TLGeoChatMessage : public TLObject, public GeoChatMessage {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLGeoChatMessageType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLMessageMedia *media READ media)
    Q_PROPERTY(TLMessageAction *action READ action)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint32 fromId READ fromId)
    Q_PROPERTY(QString message READ message)
    Q_PROPERTY(qint32 id READ id)
    Q_PROPERTY(qint32 chatId READ chatId)

public:
    enum TLGeoChatMessageType {
        TypeGeoChatMessage = GeoChatMessage::typeGeoChatMessage,
        TypeGeoChatMessageService = GeoChatMessage::typeGeoChatMessageService,
        TypeGeoChatMessageEmpty = GeoChatMessage::typeGeoChatMessageEmpty
    };

    explicit TLGeoChatMessage(QObject *parent = 0) : 
        TLObject(parent), 
        GeoChatMessage(GeoChatMessage::typeGeoChatMessageEmpty) {
    }
    TLGeoChatMessage(const TLGeoChatMessage &other) :
        TLObject(other.parent()),
        GeoChatMessage(other.classType()) {
        setData(&other);
    }
    ~TLGeoChatMessage() {}

    void setData(const void *data) {
        GeoChatMessage *obj = (GeoChatMessage *)data;
        setClassType(obj->classType());
        setMedia(obj->media());
        setAction(obj->action());
        setDate(obj->date());
        setFromId(obj->fromId());
        setMessage(obj->message());
        setId(obj->id());
        setChatId(obj->chatId());
    }

    void setMedia(const MessageMedia &media) { m_media.setData(&media); }
    void setAction(const MessageAction &action) { m_action.setData(&action); }
    TLMessageAction *action() { return &m_action; }
    TLMessageMedia *media() { return &m_media; }

private:
    TLMessageMedia m_media;
    TLMessageAction m_action;
};
Q_DECLARE_METATYPE(TLGeoChatMessage)

#endif // TLGEOCHATMESSAGE_H
