/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLGEOPOINT_H
#define TLGEOPOINT_H

#include "tlobject.h"
#include <types/geopoint.h>

class TLGeoPoint : public TLObject, public GeoPoint {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLGeoPointType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qreal lat READ lat)
    Q_PROPERTY(qreal longitude READ longitude)

public:
    enum TLGeoPointType {
        TypeGeoPoint = GeoPoint::typeGeoPoint,
        TypeGeoPointEmpty = GeoPoint::typeGeoPointEmpty
    };

    explicit TLGeoPoint(QObject *parent = 0) : 
        TLObject(parent), 
        GeoPoint(GeoPoint::typeGeoPointEmpty) {
    }
    TLGeoPoint(const TLGeoPoint &other) :
        TLObject(other.parent()),
        GeoPoint(other.classType()) {
        setData(&other);
    }
    ~TLGeoPoint() {}

    void setData(const void *data) {
        GeoPoint *obj = (GeoPoint *)data;
        setClassType(obj->classType());
        setLat(obj->lat());
        setLongitude(obj->longitude());
    }
};
Q_DECLARE_METATYPE(TLGeoPoint)

#endif // TLGEOPOINT_H
