/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLINPUTCONTACT_H
#define TLINPUTCONTACT_H

#include "tlobject.h"
#include <types/inputcontact.h>

class TLInputContact : public TLObject, public InputContact {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLInputContactType)
    Q_PROPERTY(QString firstName READ firstName WRITE setFirstName)
    Q_PROPERTY(QString lastName READ lastName WRITE setLastName)
    Q_PROPERTY(QString phone READ phone WRITE setPhone)
    Q_PROPERTY(qint64 clientId READ clientId WRITE setClientId)

public:
    enum TLInputContactType {
        TypeInputPhoneContact = InputContact::typeInputPhoneContact
    };

    explicit TLInputContact(QObject *parent = 0) :
        TLObject(parent), 
        InputContact() {
    }
    TLInputContact(const TLInputContact &other) :
        TLObject(other.parent()),
        InputContact() {
        setData(&other);
    }
    ~TLInputContact() {}

    void setClassType(qint32 classType) {
        InputContact::setClassType((InputContactType)classType);
    }
    void setData(const void *data) {
        InputContact *obj = (InputContact *)data;
        setClassType(obj->classType());
        setFirstName(obj->firstName());
        setLastName(obj->lastName());
        setPhone(obj->phone());
        setClientId(obj->clientId());
    }
};
Q_DECLARE_METATYPE(TLInputContact)

#endif // TLINPUTCONTACT_H
