/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLINPUTGEOPOINT_H
#define TLINPUTGEOPOINT_H

#include "tlobject.h"
#include <types/inputgeopoint.h>

class TLInputGeoPoint : public TLObject, public InputGeoPoint {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLInputGeoPointType)
    Q_PROPERTY(qint32 classType READ classType WRITE setClassType)
    Q_PROPERTY(qreal lat READ lat WRITE setLat)
    Q_PROPERTY(qreal longitude READ longitude WRITE setLongitude)

public:
    enum TLInputGeoPointType {
        TypeInputGeoPointEmpty = InputGeoPoint::typeInputGeoPointEmpty,
        TypeInputGeoPoint = InputGeoPoint::typeInputGeoPoint
    };

    explicit TLInputGeoPoint(QObject *parent = 0) : 
        TLObject(parent),
        InputGeoPoint(InputGeoPoint::typeInputGeoPointEmpty) {
    }
    TLInputGeoPoint(const TLInputGeoPoint &other) :
        TLObject(other.parent()),
        InputGeoPoint(other.classType()) {
        setData(&other);
    }
    ~TLInputGeoPoint() {}

    void setClassType(qint32 classType) {
        InputGeoPoint::setClassType((InputGeoPointType)classType);
    }

    void setData(const void *data) {
        InputGeoPoint *obj = (InputGeoPoint *)data;
        setClassType(obj->classType());
        setLat(obj->lat());
        setLongitude(obj->longitude());
    }
};
Q_DECLARE_METATYPE(TLInputGeoPoint)

#endif // TLINPUTGEOPOINT_H
