/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLINPUTPEER_H
#define TLINPUTPEER_H

#include "tlobject.h"
#include <types/inputpeer.h>

class TLInputPeer : public TLObject, public InputPeer {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLInputPeerType)
    Q_PROPERTY(qint32 classType READ classType WRITE setClassType NOTIFY classTypeChanged)
    Q_PROPERTY(qint64 accessHash READ accessHash WRITE setAccessHash)
    Q_PROPERTY(qint32 userId READ userId WRITE setUserId)
    Q_PROPERTY(qint32 chatId READ chatId WRITE setChatId)

public:
    enum TLInputPeerType {
        TypeInputPeerForeign = InputPeer::typeInputPeerForeign,
        TypeInputPeerSelf = InputPeer::typeInputPeerSelf,
        TypeInputPeerContact = InputPeer::typeInputPeerContact,
        TypeInputPeerEmpty = InputPeer::typeInputPeerEmpty,
        TypeInputPeerChat = InputPeer::typeInputPeerChat
    };

    explicit TLInputPeer(QObject *parent = 0) :
        TLObject(parent),
        InputPeer(InputPeer::typeInputPeerEmpty) {
    }
    TLInputPeer(const TLInputPeer &other) :
        TLObject(other.parent()),
        InputPeer(other.classType()) {
        setData(&other);
    }
    ~TLInputPeer() {}

    void setClassType(qint32 classType) {
        InputPeer::setClassType((InputPeer::InputPeerType)classType);
        Q_EMIT classTypeChanged(classType);
    }

    void setData(const void *data) {
        InputPeer *obj = (InputPeer *)data;
        setClassType(obj->classType());
        setAccessHash(obj->accessHash());
        setUserId(obj->userId());
        setChatId(obj->chatId());
    }

Q_SIGNALS:
    void classTypeChanged(qint32);
};
Q_DECLARE_METATYPE(TLInputPeer)

#endif // TLINPUTPEER_H
