/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLINPUTUSER_H
#define TLINPUTUSER_H

#include "tlobject.h"
#include <types/inputuser.h>

class TLInputUser : public TLObject, public InputUser {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLInputUserType)
    Q_PROPERTY(qint32 classType READ classType WRITE setClassType)
    Q_PROPERTY(qint64 accessHash READ accessHash WRITE setAccessHash)
    Q_PROPERTY(qint32 userId READ userId WRITE setUserId)

public:
    enum TLInputUserType {
        TypeInputUserEmpty = InputUser::typeInputUserEmpty,
        TypeInputUserContact = InputUser::typeInputUserContact,
        TypeInputUserSelf = InputUser::typeInputUserSelf,
        TypeInputUserForeign = InputUser::typeInputUserForeign
    };

    explicit TLInputUser(QObject *parent = 0) : 
        TLObject(parent), 
        InputUser(InputUser::typeInputUserEmpty) {
    }
    TLInputUser(const TLInputUser &other) :
        TLObject(other.parent()),
        InputUser(other.classType()) {
        setData(&other);
    }
    ~TLInputUser() {}

    void setClassType(qint32 classType) {
        InputUser::setClassType((InputUserType)classType);
    }
    void setData(const void *data) {
        InputUser *obj = (InputUser *)data;
        setClassType(obj->classType());
        setAccessHash(obj->accessHash());
        setUserId(obj->userId());
    }
};
Q_DECLARE_METATYPE(TLInputUser)

#endif // TLINPUTUSER_H
