/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLMESSAGEACTION_H
#define TLMESSAGEACTION_H

#include "tlobject.h"
#include "tlphoto.h"
#include <types/messageaction.h>
#include <QQmlListProperty>

class TLMessageAction : public TLObject, public MessageAction {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLMessageActionType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLPhoto *photo READ photo)
    Q_PROPERTY(QString address READ address)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(QList<qint32> users READ users)
    Q_PROPERTY(QString title READ title)

public:
    enum TLMessageActionType {
        TypeMessageActionEmpty = MessageAction::typeMessageActionEmpty,
        TypeMessageActionChatDeletePhoto = MessageAction::typeMessageActionChatDeletePhoto,
        TypeMessageActionChatCreate = MessageAction::typeMessageActionChatCreate,
        TypeMessageActionChatEditTitle = MessageAction::typeMessageActionChatEditTitle,
        TypeMessageActionChatEditPhoto = MessageAction::typeMessageActionChatEditPhoto,
        TypeMessageActionGeoChatCreate = MessageAction::typeMessageActionGeoChatCreate,
        TypeMessageActionChatDeleteUser = MessageAction::typeMessageActionChatDeleteUser,
        TypeMessageActionChatAddUser = MessageAction::typeMessageActionChatAddUser,
        TypeMessageActionGeoChatCheckin = MessageAction::typeMessageActionGeoChatCheckin
    };

    explicit TLMessageAction(QObject *parent = 0) : 
        TLObject(parent), 
        MessageAction(MessageAction::typeMessageActionEmpty) {
    }
    TLMessageAction(const TLMessageAction &other) :
        TLObject(other.parent()),
        MessageAction(other.classType()) {
        setData(&other);
    }
    ~TLMessageAction() {}

    void setData(const void *data) {
        MessageAction *obj = (MessageAction *)data;
        setClassType(obj->classType());
        setPhoto(obj->photo());
        setAddress(obj->address());
        setUserId(obj->userId());
        setUsers(obj->users());
        setTitle(obj->title());
    }

    void setPhoto(const Photo &photo) { m_photo.setData(&photo); }
    TLPhoto *photo() { return &m_photo; }

private:
    TLPhoto m_photo;
};
Q_DECLARE_METATYPE(TLMessageAction)
#endif // TLMESSAGEACTION_H
