/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLMESSAGEMEDIA_H
#define TLMESSAGEMEDIA_H

#include "tlobject.h"
#include "tlaudio.h"
#include "tlphoto.h"
#include "tlvideo.h"
#include "tldocument.h"
#include <types/messagemedia.h>

class TLMessageMedia : public TLObject, public MessageMedia {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLMessageMediaType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLAudio *audio READ audio)
    Q_PROPERTY(QString firstName READ firstName)
    Q_PROPERTY(QString lastName READ lastName)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(QByteArray bytes READ bytes)
    Q_PROPERTY(TLVideo *video READ video)
    Q_PROPERTY(TLPhoto *photo READ photo)
    Q_PROPERTY(QString phoneNumber READ phoneNumber)
    Q_PROPERTY(TLDocument *document READ document)
    Q_PROPERTY(TLGeoPoint *geo READ geo)

public:
    enum TLMessageMediaType {
        TypeMessageMediaDocument = MessageMedia::typeMessageMediaDocument,
        TypeMessageMediaContact = MessageMedia::typeMessageMediaContact,
        TypeMessageMediaEmpty = MessageMedia::typeMessageMediaEmpty,
        TypeMessageMediaVideo = MessageMedia::typeMessageMediaVideo,
        TypeMessageMediaUnsupported = MessageMedia::typeMessageMediaUnsupported,
        TypeMessageMediaAudio = MessageMedia::typeMessageMediaAudio,
        TypeMessageMediaPhoto = MessageMedia::typeMessageMediaPhoto,
        TypeMessageMediaGeo = MessageMedia::typeMessageMediaGeo
    };

    explicit TLMessageMedia(QObject *parent = 0) : 
        TLObject(parent), 
        MessageMedia(MessageMedia::typeMessageMediaEmpty) {
    }
    TLMessageMedia(const TLMessageMedia &other) :
        TLObject(other.parent()),
        MessageMedia(other.classType()) {
        setData(&other);
    }
    ~TLMessageMedia() {}

    void setData(const void *data) {
        MessageMedia *obj = (MessageMedia *)data;
        setClassType(obj->classType());
        setAudio(obj->audio());
        setFirstName(obj->firstName());
        setLastName(obj->lastName());
        setUserId(obj->userId());
        setBytes(obj->bytes());
        setVideo(obj->video());
        setPhoto(obj->photo());
        setPhoneNumber(obj->phoneNumber());
        setDocument(obj->document());
        setGeo(obj->geo());
    }

    void setAudio(const Audio &audio) { m_audio.setData(&audio); }
    void setVideo(const Video &video) { m_video.setData(&video); }
    void setPhoto(const Photo &photo) { m_photo.setData(&photo); }
    void setDocument(const Document &document) { m_document.setData(&document); }
    void setGeo(const GeoPoint &geo) { m_geo.setData(&geo); }
    TLAudio *audio() { return &m_audio; }
    TLVideo *video() { return &m_video; }
    TLPhoto *photo() { return &m_photo; }
    TLDocument *document() { return &m_document; }
    TLGeoPoint *geo() { return &m_geo; }

private:
    TLAudio m_audio;
    TLVideo m_video;
    TLPhoto m_photo;
    TLDocument m_document;
    TLGeoPoint m_geo;

};
Q_DECLARE_METATYPE(TLMessageMedia)

#endif // TLMESSAGEMEDIA_H
