/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLNOTIFYPEER_H
#define TLNOTIFYPEER_H

#include "tlobject.h"
#include "tlpeer.h"
#include <types/notifypeer.h>

class TLNotifyPeer : public TLObject, public NotifyPeer {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLNotifyPeerType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLPeer *peer READ peer)

public:
    enum TLNotifyPeerType {
        TypeNotifyAll = NotifyPeer::typeNotifyAll,
        TypeNotifyPeer = NotifyPeer::typeNotifyPeer,
        TypeNotifyUsers = NotifyPeer::typeNotifyUsers,
        TypeNotifyChats = NotifyPeer::typeNotifyChats
    };

    explicit TLNotifyPeer(QObject *parent = 0) : 
        TLObject(parent), 
        NotifyPeer(NotifyPeer::typeNotifyAll) {
    }
    TLNotifyPeer(const TLNotifyPeer &other) :
        TLObject(other.parent()),
        NotifyPeer(other.classType()) {
        setData(&other);
    }
    ~TLNotifyPeer() {}

    void setData(const void *data) {
        NotifyPeer *obj = (NotifyPeer *)data;
        setClassType(obj->classType());
        setPeer(obj->peer());
    }

    void setPeer(const Peer &peer) { m_peer.setData(&peer); }
    TLPeer *peer() { return &m_peer; }

private:
    TLPeer m_peer;
};
Q_DECLARE_METATYPE(TLNotifyPeer)

#endif // TLNOTIFYPEER_H
