/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLPEER_H
#define TLPEER_H

#include <types/peer.h>
#include "tlobject.h"

class TLPeer : public TLObject, public Peer {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLPeerType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(qint32 chatId READ chatId)

public:
    enum TLPeerType {
       TypePeerUser = Peer::typePeerUser,
       TypePeerChat = Peer::typePeerChat
    };

    explicit TLPeer(QObject *parent = 0) : 
        TLObject(parent), 
        Peer(Peer::typePeerUser) {
    }
    TLPeer(const TLPeer &other) :
        TLObject(other.parent()),
        Peer(other.classType()) {
        setData(&other);
    }
    ~TLPeer() {}

    void setData(const void *data) {
        Peer *obj = (Peer *)data;
        setClassType(obj->classType());
        setUserId(obj->userId());
        setChatId(obj->chatId());
    }
};
Q_DECLARE_METATYPE(TLPeer)
#endif // TLPEER_H
