/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLPEERNOTIFYSETTINGS_H
#define TLPEERNOTIFYSETTINGS_H

#include "tlobject.h"
#include <types/peernotifysettings.h>

class TLPeerNotifySettings : public TLObject, public PeerNotifySettings {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLPeerNotifySettingsType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 muteUntil READ muteUntil)
    Q_PROPERTY(QString sound READ sound)
    Q_PROPERTY(bool showPreviews READ showPreviews)
    Q_PROPERTY(qint32 eventsMask READ eventsMask)

public:
    enum TLPeerNotifySettingsType {
        TypePeerNotifySettings = PeerNotifySettings::typePeerNotifySettings,
        TypePeerNotifySettingsEmpty = PeerNotifySettings::typePeerNotifySettingsEmpty
    };

    explicit TLPeerNotifySettings(QObject *parent = 0) : 
        TLObject(parent), 
        PeerNotifySettings(PeerNotifySettings::typePeerNotifySettingsEmpty) {
    }
    TLPeerNotifySettings(const TLPeerNotifySettings &other) :
        TLObject(other.parent()),
        PeerNotifySettings(other.classType()) {
        setData(&other);
    }
    ~TLPeerNotifySettings() {}

    void setData(const void *data) {
        PeerNotifySettings *obj = (PeerNotifySettings *)data;
        setClassType(obj->classType());
        setMuteUntil(obj->muteUntil());
        setSound(obj->sound());
        setShowPreviews(obj->showPreviews());
        setEventsMask(obj->eventsMask());
    }
};
Q_DECLARE_METATYPE(TLPeerNotifySettings)

#endif // TLPEERNOTIFYSETTINGS_H
