/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLPHOTO_H
#define TLPHOTO_H

#include "tlobject.h"
#include "tlgeopoint.h"
#include "tlphotosize.h"
#include <types/photo.h>
#include <types/photosize.h>
#include <types/geopoint.h>
#include <QQmlListProperty>

class TLPhoto : public TLObject, public Photo {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLPhotoType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(QQmlListProperty<TLPhotoSize> sizes READ sizes)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(QString caption READ caption)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(TLGeoPoint *geo READ geo)
    Q_PROPERTY(qint64 id READ id)

public:
    enum TLPhotoType {
        TypePhoto = Photo::typePhoto,
        TypePhotoEmpty = Photo::typePhotoEmpty
    };

    explicit TLPhoto(QObject *parent = 0) : 
        TLObject(parent), 
        Photo(Photo::typePhotoEmpty) {
    }
    TLPhoto(const TLPhoto &other) :
        TLObject(other.parent()),
        Photo(other.classType()) {
        setData(&other);
    }
    ~TLPhoto() {
        Q_FOREACH (TLPhotoSize *size, m_sizes) {
            if (size) {
                delete size;
                size = 0;
            }
        }
    }

    void setData(const void *data) {
        Photo *obj = (Photo *)data;
        setClassType(obj->classType());
        setAccessHash(obj->accessHash());
        setUserId(obj->userId());
        setCaption(obj->caption());
        setDate(obj->date());
        setGeo(obj->geo());
        setId(obj->id());
        Q_FOREACH (PhotoSize s, obj->sizes()) {
            TLPhotoSize *tlPhotoSize = new TLPhotoSize(this);
            tlPhotoSize->setData(&s);
            m_sizes.append(tlPhotoSize);
        }
    }

    QQmlListProperty<TLPhotoSize> sizes() {
        return QQmlListProperty<TLPhotoSize>(this, m_sizes);
    }

    void setGeo(const GeoPoint &geo) { m_geo.setData(&geo); }
    TLGeoPoint *geo() { return &m_geo; }

private:
    QList<TLPhotoSize *> m_sizes;
    TLGeoPoint m_geo;
};
Q_DECLARE_METATYPE(TLPhoto)

#endif // TLPHOTO_H
