/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLPHOTOSIZE_H
#define TLPHOTOSIZE_H

#include "tlobject.h"
#include "tlfilelocation.h"
#include <types/photosize.h>

class TLPhotoSize : public TLObject, public PhotoSize {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLPhotoSizeType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 h READ h)
    Q_PROPERTY(QByteArray bytes READ bytes)
    Q_PROPERTY(TLFileLocation *location READ location)
    Q_PROPERTY(qint32 w READ w)
    Q_PROPERTY(QString type READ type)
    Q_PROPERTY(qint32 size READ size)

public:
    enum TLPhotoSizeType {
        TypePhotoCachedSize = PhotoSize::typePhotoCachedSize,
        TypePhotoSizeEmpty = PhotoSize::typePhotoSizeEmpty,
        TypePhotoSize = PhotoSize::typePhotoSize
    };

    explicit TLPhotoSize(QObject *parent = 0) : 
        TLObject(parent), 
        PhotoSize(PhotoSize::typePhotoSizeEmpty) {
    }
    TLPhotoSize(const TLPhotoSize &other) :
        TLObject(other.parent()),
        PhotoSize(other.classType()) {
        setData(&other);
    }
    ~TLPhotoSize() {}

    void setData(const void *data) {
        PhotoSize *obj = (PhotoSize *)data;
        setClassType(obj->classType());
        setH(obj->h());
        setBytes(obj->bytes());
        setLocation(obj->location());
        setW(obj->w());
        setType(obj->type());
        setSize(obj->size());
    }

    void setLocation(const FileLocation &location) { m_location.setData(&location); }
    TLFileLocation *location() { return &m_location; }

private:
    TLFileLocation m_location;
};
Q_DECLARE_METATYPE(TLPhotoSize)

#endif // TLPHOTOSIZE_H
