/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLSTORAGEFILETYPE_H
#define TLSTORAGEFILETYPE_H

#include "tlobject.h"
#include <types/storagefiletype.h>

class TLStorageFileType : public TLObject, public StorageFileType {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLStorageFileTypeType)
    Q_PROPERTY(qint32 classType READ classType)

public:
    enum TLStorageFileTypeType {
        TypeStorageFileMov = StorageFileType::typeStorageFileMov,
        TypeStorageFilePartial = StorageFileType::typeStorageFilePartial,
        TypeStorageFileUnknown = StorageFileType::typeStorageFileUnknown,
        TypeStorageFileJpeg = StorageFileType::typeStorageFileJpeg,
        TypeStorageFilePdf = StorageFileType::typeStorageFilePdf,
        TypeStorageFileMp4 = StorageFileType::typeStorageFileMp4,
        TypeStorageFilePng = StorageFileType::typeStorageFilePng,
        TypeStorageFileGif = StorageFileType::typeStorageFileGif,
        TypeStorageFileMp3 = StorageFileType::typeStorageFileMp3,
        TypeStorageFileWebp = StorageFileType::typeStorageFileWebp
    };

    explicit TLStorageFileType(QObject *parent = 0) : 
        TLObject(parent), 
        StorageFileType(StorageFileType::typeStorageFileUnknown) {
    }
    TLStorageFileType(const TLStorageFileType &other) :
        TLObject(other.parent()),
        StorageFileType(other.classType()) {
        setData(&other);
    }
    ~TLStorageFileType() {}

    void setClassType(qint32 classType) {
        StorageFileType::setClassType((StorageFileTypeType)classType);
    }
    void setData(const void *data) {
        StorageFileType *obj = (StorageFileType *)data;
        setClassType(obj->classType());
    }
};
Q_DECLARE_METATYPE(TLStorageFileType)

#endif // TLSTORAGEFILETYPE_H
