/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLUSER_H
#define TLUSER_H

#include <types/types.h>
#include "tlobject.h"
#include "tluserstatus.h"
#include "tluserprofilephoto.h"

class TLUser : public TLObject, public User {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLUserType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(const TLUserStatus *status READ status)
    Q_PROPERTY(QString firstName READ firstName)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(QString lastName READ lastName)
    Q_PROPERTY(QString phone READ phone)
    Q_PROPERTY(bool inactive READ inactive)
    Q_PROPERTY(const TLUserProfilePhoto *photo READ photo)
    Q_PROPERTY(qint32 id READ id)
public:
    enum TLUserType {
        TypeUserSelf = User::typeUserSelf,
        TypeUserContact = User::typeUserContact,
        TypeUserDeleted = User::typeUserDeleted,
        TypeUserForeign = User::typeUserForeign,
        TypeUserEmpty = User::typeUserEmpty,
        TypeUserRequest = User::typeUserRequest
    };

    explicit TLUser(QObject *parent = 0) :
        TLObject(parent),
        User(User::typeUserEmpty) {
    }
    TLUser(const TLUser &other) :
        TLObject(other.parent()),
        User(other.classType()) {
        setData(&other);
    }
    ~TLUser() {}

    void setData(const void *data) {
        User *u = (User*)data;
        setId(u->id());
        setAccessHash(u->accessHash());
        setInactive(u->inactive());
        setPhone(u->phone());
        setFirstName(u->firstName());
        setLastName(u->lastName());
        setPhoto(u->photo());
        setStatus(u->status());
        setClassType(u->classType());
    }

    void setPhoto(const UserProfilePhoto &photo) { m_photo.setData(&photo); }
    void setStatus(const UserStatus &status) { m_status.setData(&status); }
    const TLUserStatus *status() const { return &m_status; }
    const TLUserProfilePhoto *photo() const { return &m_photo; }

private:
    TLUserStatus m_status;
    TLUserProfilePhoto m_photo;
};
Q_DECLARE_METATYPE(TLUser)

#endif // TLUSER_H
