/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLUSERPROFILEPHOTO_H
#define TLUSERPROFILEPHOTO_H

#include "tlobject.h"
#include "tlfilelocation.h"
#include <types/userprofilephoto.h>

class TLUserProfilePhoto : public TLObject, public UserProfilePhoto {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLUserProfilePhotoType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(TLFileLocation *photoSmall READ photoSmall)
    Q_PROPERTY(qint64 photoId READ photoId)
    Q_PROPERTY(TLFileLocation *photoBig READ photoBig)

public:
    enum TLUserProfilePhotoType {
        TypeUserProfilePhoto = UserProfilePhoto::typeUserProfilePhoto,
        TypeUserProfilePhotoEmpty = UserProfilePhoto::typeUserProfilePhotoEmpty
    };

    explicit TLUserProfilePhoto(QObject *parent = 0) : 
        TLObject(parent), 
        UserProfilePhoto(UserProfilePhoto::typeUserProfilePhotoEmpty) {
    }
    TLUserProfilePhoto(const TLUserProfilePhoto &other) :
        TLObject(other.parent()),
        UserProfilePhoto(other.classType()) {
        setData(&other);
    }
    ~TLUserProfilePhoto() {}

    void setData(const void *data) {
        UserProfilePhoto *obj = (UserProfilePhoto *)data;
        setClassType(obj->classType());
        setPhotoSmall(obj->photoSmall());
        setPhotoId(obj->photoId());
        setPhotoBig(obj->photoBig());
    }

    void setPhotoSmall(const FileLocation &photoSmall) { m_photoSmall.setData(&photoSmall); }
    void setPhotoBig(const FileLocation &photoBig) { m_photoBig.setData(&photoBig); }
    TLFileLocation *photoSmall() { return &m_photoSmall; }
    TLFileLocation *photoBig() { return &m_photoBig; }

private:
    TLFileLocation m_photoSmall;
    TLFileLocation m_photoBig;
};
Q_DECLARE_METATYPE(TLUserProfilePhoto)

#endif // TLUSERPROFILEPHOTO_H
