/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLUSERSTATUS_H
#define TLUSERSTATUS_H

#include "tlobject.h"
#include <types/userstatus.h>

class TLUserStatus : public TLObject, public UserStatus {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLUserStatusType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(qint32 expires READ expires)
    Q_PROPERTY(qint32 wasOnline READ wasOnline)

public:
    enum TLUserStatusType {
        TypeUserStatusOffline = UserStatus::typeUserStatusOffline,
        TypeUserStatusEmpty = UserStatus::typeUserStatusEmpty,
        TypeUserStatusOnline = UserStatus::typeUserStatusOnline
    };

    explicit TLUserStatus(QObject *parent = 0) : 
        TLObject(parent), 
        UserStatus(UserStatus::typeUserStatusEmpty) {
    }
    TLUserStatus(const TLUserStatus &other) :
        TLObject(other.parent()),
        UserStatus(other.classType()) {
        setData(&other);
    }
    ~TLUserStatus() {}

    void setData(const void *data) {
        UserStatus *obj = (UserStatus *)data;
        setClassType(obj->classType());
        setExpires(obj->expires());
        setWasOnline(obj->wasOnline());
    }
};
Q_DECLARE_METATYPE(TLUserStatus)

#endif // TLUSERSTATUS_H
