/*
 * Copyright 2014 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef TLVIDEO_H
#define TLVIDEO_H

#include "tlobject.h"
#include "tlphotosize.h"
#include <types/video.h>

class TLVideo : public TLObject, public Video {
    Q_OBJECT
    Q_INTERFACES(TLObject)
    Q_ENUMS(TLVideoType)
    Q_PROPERTY(qint32 classType READ classType)
    Q_PROPERTY(QString mimeType READ mimeType)
    Q_PROPERTY(TLPhotoSize *thumb READ thumb)
    Q_PROPERTY(QString caption READ caption)
    Q_PROPERTY(qint64 accessHash READ accessHash)
    Q_PROPERTY(qint32 dcId READ dcId)
    Q_PROPERTY(qint32 userId READ userId)
    Q_PROPERTY(qint32 duration READ duration)
    Q_PROPERTY(qint32 w READ w)
    Q_PROPERTY(qint32 h READ h)
    Q_PROPERTY(qint32 date READ date)
    Q_PROPERTY(qint64 id READ id)
    Q_PROPERTY(qint32 size READ size)

public:
    enum TLVideoType {
        TypeVideoEmpty = Video::typeVideoEmpty,
        TypeVideo = Video::typeVideo
    };

    explicit TLVideo(QObject *parent = 0) : 
        TLObject(parent), 
        Video(Video::typeVideoEmpty) {
    }
    TLVideo(const TLVideo &other) :
        TLObject(other.parent()),
        Video(other.classType()) {
        setData(&other);
    }
    ~TLVideo() {}

    void setData(const void *data) {
        Video *obj = (Video *)data;
        setClassType(obj->classType());
        setMimeType(obj->mimeType());
        setThumb(obj->thumb());
        setCaption(obj->caption());
        setAccessHash(obj->accessHash());
        setDcId(obj->dcId());
        setUserId(obj->userId());
        setDuration(obj->duration());
        setW(obj->w());
        setH(obj->h());
        setDate(obj->date());
        setId(obj->id());
        setSize(obj->size());
    }

    void setThumb(const PhotoSize &thumb) { m_thumb.setData(&thumb); }
    TLPhotoSize *thumb() { return &m_thumb; }

private:
    TLPhotoSize m_thumb;
};
Q_DECLARE_METATYPE(TLVideo)

#endif // TLVIDEO_H
